*****************************************************************************
*                                                                           *
* Single Chip Chess Computer (SC3) for Motorola 68HC705C8 Microcontroller   *
*                                                                           *
*                           15 May 1993                                     *
*                                                                           *
*                     Copyright (C) 1993,1994                               *
*                                                                           *
*                          Ian Mitchell                                     *
*                                                                           *
*                     WALTRUD SYSTEMS TASMANIA                              *
*                                                                           *
*****************************************************************************

*****************************************************************************
*                                                                           *
* This is the header file.                                                  *
*                                                                           *
*****************************************************************************

ram1    	equ     $0030
ram2    	equ     $0100
eprom   	equ     $0160
lb              equ	$a0		offset for indexing stack elements
SEARCH_STACK    equ	8		maximum size of search stack
BOARD           equ	64		number of bytes required for the board
OP_MOVES	equ	10		number of opening moves

		org     ram1
b       	rmb     BOARD   	the board
sb		rmb	BOARD		saved board, permits interrupt at any time
l       	rmb     1       	stack level
Ckp     	rmb     1       	current computers Kings position
Okp     	rmb     1       	current opponents Kings position
cf      	rmb     1       	flag indicates castling is a legal move
temp     	rmb     1       	temporary saved from piece and other things
temp2     	rmb     1       	temporary storage
temp3     	rmb     1       	temporary storage
temp4     	rmb     1       	temporary storage
temp5     	rmb     1       	temporary storage
temp6     	rmb     1       	temporary storage
temp7     	rmb     1       	temporary storage
temp8     	rmb     1       	temporary storage
s_Okp		rmb	1		saved opponents king position
v       	rmb     1       	value so far
bal     	rmb     1       	current material balance
dir     	rmb     1       	current piece direction
frm     	rmb     1       	FROM position
pto     	rmb     1       	TO position
stp     	rmb     1       	captured piece
mf		rmb	1		special move flag
Cmat    	rmb     1       	computers current material
Omat    	rmb     1       	opponents current material
level   	rmb     1       	opponent selected difficulty of play
to_level   	rmb  	1       	current depth iteratively deepening
half_moves 	rmb  	1       	number of half moves from beginning of game
pcf0    	rmb     1       	principle continuation (from) level 0
pct0    	rmb     1       	principle continuation (to)   level 0
bcf		rmb	1		saved castle flag from best move
bkp		rmb	1		saved king position from best move
mflag   	rmb     1       	flag indicates mates or draw
mp		rmb	1		positional score
mb		rmb	1		best positional score so far
multiplex	rmb	1		current LED group on display
fled		rmb	1		interval counter for flasing 'searching' LED
O_white		rmb	1		indicates opponent is playing white pieces
fromto		rmb	1		parameter to show_opp routine
gametime_hi	rmb	1		high byte of time remaining for search
gametime_lo	rmb	1		low byte of time remaining for search
totaltime_hi	rmb	1		high byte of max number of seconds of searching
totaltime_lo	rmb	1		low byte of max number of seconds of searching
sleeping	rmb	1		true (none zero) if processor has stopped
flash_grp	rmb	1		indicates which group of LEDs should flash
l_frm		rmb	1		FROM from last iteration
l_pto		rmb	1		TO from last iteration
l_score		rmb	1		score from last iteration
*fl_rate	equ	5		flash rate (an Nth power of 2 matched
fl_rate		equ	6		flash rate (an Nth power of 2 matched
*					to the multiplex loop count)
i_frm		equ	temp2
i_pto		equ	temp3
i_mf		equ	temp4
i_cap		equ	temp5
i_cf		equ	temp6
i_kp		equ	temp7

		org     ram2
sdir0    	rmb     SEARCH_STACK	move direction
sfrm0_1    	rmb     1		this is frm-1
sfrm0    	rmb     SEARCH_STACK    board index of piece to move
spto0_1    	rmb     1		this is pto-1
spto0    	rmb     SEARCH_STACK    board index to move piece to
sstp0    	rmb     SEARCH_STACK    saved captured piece (or blank)
sskp0    	rmb     SEARCH_STACK    saved king position
sscf0_2		rmb	1		this scf-2
sscf0_1		rmb	1		this scf-1
sscf0    	rmb     SEARCH_STACK    saved castle ok flag
scap0    	rmb     SEARCH_STACK    flag, true if considering captures
ssmf0    	rmb     SEARCH_STACK    special move flag
a0_1     	rmb     1       	this is a-1
a0       	rmb     SEARCH_STACK    alpha beta stack values
m0		rmb	SEARCH_STACK	tempory min max value
sdir    	equ     sdir0-lb	move direction
sfrm_1    	equ     sfrm0-1-lb	this is sfrm-1
sfrm    	equ     sfrm0-lb	board index of piece to move
spto_1    	equ     spto0-1-lb	this is spto-1
spto    	equ     spto0-lb	board index to move piece to
sstp    	equ     sstp0-lb	saved captured piece (or blank)
sskp    	equ     sskp0-lb	saved king position
sscf_2		equ	sscf0-2-lb 	this is sscf-2
sscf_1		equ	sscf0-1-lb 	this is sscf-1
sscf    	equ     sscf0-lb	saved castle ok flag
scap    	equ     scap0-lb	flag, true if considering captures
ssmf    	equ     ssmf0-lb	special moves flag
a_2		equ	a0-2-lb		this is a-2
a_1     	equ     a0_1-lb		this is a-1
a       	equ     a0-lb		alpha beta stack values
m		equ	m0-lb		tempory min max value
dir0    	equ     sdir    	move direction
frm0    	equ     sfrm    	board index of piece to move
pto0    	equ     spto   		board index to move piece to
stp0    	equ     sstp    	saved captured piece (or blank)
skp0    	equ     sskp    	saved king position
scf0    	equ     sscf    	saved castle ok flag
BPOS0		equ	b		address of board location 0
BPOS1		equ	b+1		address of board location 1
BPOS2		equ	b+2		address of board location 2
BPOS3		equ	b+3		address of board location 3
BPOS4		equ	b+4		address of board location 4
BPOS5		equ	b+5		address of board location 5
BPOS6		equ	b+6		address of board location 6
BPOS7		equ	b+7		address of board location 7
BPOS56		equ	b+56		address of board location 56
BPOS57		equ	b+57		address of board location 57
BPOS58		equ	b+58		address of board location 58
BPOS59		equ	b+59		address of board location 59
BPOS60		equ	b+60		address of board location 60
BPOS61		equ	b+61		address of board location 61
BPOS62		equ	b+62		address of board location 62
BPOS63		equ	b+63		address of board location 63
SBPOS0		equ	sb		address of saved board location 0
SBPOS1		equ	sb+1		address of saved board location 1
SBPOS2		equ	sb+2		address of saved board location 2
SBPOS3		equ	sb+3		address of saved board location 3
SBPOS4		equ	sb+4		address of saved board location 4
SBPOS5		equ	sb+5		address of saved board location 5
SBPOS6		equ	sb+6		address of saved board location 6
SBPOS7		equ	sb+7		address of saved board location 7
SBPOS56		equ	sb+56		address of saved board location 56
SBPOS57		equ	sb+57		address of saved board location 57
SBPOS58		equ	sb+58		address of saved board location 58
SBPOS59		equ	sb+59		address of saved board location 59
SBPOS60		equ	sb+60		address of saved board location 60
SBPOS61		equ	sb+61		address of saved board location 61
SBPOS62		equ	sb+62		address of saved board location 62
SBPOS63		equ	sb+63		address of saved board location 63

maxscore	equ	63		this is the maximum score
minscore	equ	-63		this is the minimum score
mate		equ	1       	check and mate
draw	   	equ	2       	draw or stale mate
know_move	equ	4		an exact score

C_pawn		equ	1
C_knight	equ	3
C_rook		equ	5
C_bishop	equ	4
C_queen		equ	9
C_king		equ	6

O_pawn		equ	-1
O_knight	equ	-3
O_rook		equ	-5
O_bishop	equ	-4
O_queen		equ	-9
O_king		equ	-6

pawn		equ	1
knight		equ	3
rook		equ	5
bishop		equ	4
queen		equ	9
king		equ	6
blank		equ	0
promotion	equ	-1
ep		equ	1
debounce	equ	14		num multiplex loops to debounce key
flash		equ	16		count of 63.5 ms intervals after which
*                                       the progress (MATE) LED is flashed

* register equates
PORTA           EQU     $00		Port A register
PORTB           EQU     $01             Port B register
PORTC           EQU     $02             Port C register
PORTD           EQU     $03             Port D register
DDRA            EQU     $04             Data Direction Register A
DDRB            EQU     $05             Data Direction Register B
DDRC            EQU     $06             Data Direction Register C
SPCR            EQU     $0A             SPIE,SPE,-,MSTR;CPOL,CPHA,SPR1,SPR0
SPSR            EQU     $0B             SPIF,WCOL,-,MODF;-,-,-,-
SPDR            EQU     $0C             SPI Data
BAUD            EQU     $0D             -,-,SCP1,SCP0;-,SCR2,SCR1,SCR0
SCCR1           EQU     $0E             R8,T8,-,M;WAKE,-,-,-
SCCR2           EQU     $0F             TIE,TCIE,RIE,ILIE;TE,RE,RWU,SBK
SCSR            EQU     $10             TDRE,TC,RDRF,IDLE;OR,NF,FE,-
SCDR            EQU     $11             SCI Data
RDR             EQU     $11             SCI Receive Data (same as SCDR)
TDR             EQU     $11             SCI Transmit Data (same as SCDR)
TCR             EQU     $12             ICIE,OCIE,TOIE,0;0,0,IEGE,OLVL
TSR             EQU     $13             ICF,OCF,TOF,0; 0,0,0,0
ICAPHI          EQU     $14             Input Capture Reg Hi
ICAPLOW         EQU     $15             Input Capture Reg Low
OCRHI		EQU     $16             Output Compare Reg Hi
OCRLO           EQU     $17             Output Compare Reg Low
TCNTHI          EQU     $18             Timer Count Reg Hi
TCNTLO          EQU     $19             Timer Count Reg Low
ACRHI           EQU     $1A             Alternate Count Reg Hi
ACRLO           EQU     $1B             Alternate Count Reg Low
EPR             EQU     $1C             Eprom Program Register
CRR             EQU     $1D             Cop Reset Register
CCR             EQU     $1E             Cop Control Register
OPTR		EQU	$1FDF		option register

* relations to hardware
src_port	equ	PORTA		source port for multiplexing
snk_port	equ	PORTB		sink port for multiplexing
in_port		equ	PORTC		push buttons are connected to port C

*****************************************************************************
*                                                                           *
* Board setup                                                               *
*                                                                           *
*****************************************************************************
		org	ram2
set_bd1		fcb	C_rook
		fcb	C_knight
		fcb	C_bishop
		fcb	C_queen
		fcb	C_king
		fcb	C_bishop
		fcb	C_knight
		fcb	C_rook
		fcb	C_pawn
		fcb	C_pawn
		fcb	C_pawn
		fcb	C_pawn
		fcb	C_pawn
		fcb	C_pawn
		fcb	C_pawn
		fcb	C_pawn
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
*		fcb	blank
set_bd2		fcb	O_pawn
		fcb	O_pawn
		fcb	O_pawn
		fcb	O_pawn
		fcb	O_pawn
		fcb	O_pawn
		fcb	O_pawn
		fcb	O_pawn
		fcb	O_rook
		fcb	O_knight
		fcb	O_bishop
		fcb	O_queen
		fcb	O_king
		fcb	O_bishop
		fcb	O_knight
		fcb	O_rook

*****************************************************************************
*                                                                           *
* Subroutine to set up the board and king positions                         *
*                                                                           *
*****************************************************************************
*set_up_board	ldx	#63		index the setup and game board
*set_loop	lda	set_bd,x	get a piece
*		sta	b,x		set up the board
*		decx			point to next piece
*		bpl	set_loop	until all set
*
set_up_all	clr	PORTA		clear all ports before setting
		clr	PORTB		direction of i/o
		clr	PORTC
		lda	#$ff		PORTA is an output port
		sta	DDRA
		lda	#$f		PORTB bits 0-3, output
		sta	DDRB
		clr	DDRC		PORTC is input port
reset_wait	lda	in_port		if a key is pressed
		ora	#$f8		mask to set high bits
		coma			wait for all ones 'cause input is
		bne	reset_wait	active low
		clr	sleeping	stopped flag, set false
*
* set up the board
*

		ldx	#15		index the setup and game board
set_loop	lda	set_bd1,x	get a piece
		sta	b,x		set up the board
		lda	set_bd2,x
		sta	b+48,x
		clr	b+16,x
		clr	b+32,x
		decx			point to next piece
		bpl	set_loop	until all set
*
* init kings position, castle flags, number of moves
*
		lda	#4		computers king position
		sta	Ckp
		lda	#60		opponents king position
		sta	Okp
		sta	s_Okp
		clr	half_moves
*
* init display multiplexor
*
		lda	#$ff
		sta	multiplex
		jsr	sleep0125	sleep 0.125 seconds
		rts

*****************************************************************************
*                                                                           *
* build the direction tables (directions are with respect to compters move) *
*                                                                           *
*****************************************************************************
		org     eprom
*
* dir table for moving north
*
DIRN
TOPDIR
		fcb       8
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb      15
		fcb      16
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb      23
		fcb      24
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb      31
		fcb      32
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb      39
		fcb      40
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb      47
		fcb      48
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54
		fcb      55
		fcb      56
		fcb      57
		fcb      58
		fcb      59
		fcb      60
		fcb      61
		fcb      62
		fcb      63
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255

* dir table for moving south
DIRS
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb       0
		fcb       1
		fcb       2
		fcb       3
		fcb       4
		fcb       5
		fcb       6
		fcb       7
		fcb       8
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb      15
		fcb      16
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb      23
		fcb      24
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb      31
		fcb      32
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb      39
		fcb      40
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb      47
		fcb      48
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54
		fcb      55

* dir table for moving east
DIRE
		fcb       1
		fcb       2
		fcb       3
		fcb       4
		fcb       5
		fcb       6
		fcb       7
		fcb     255
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb      15
		fcb     255
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb      23
		fcb     255
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb      31
		fcb     255
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb      39
		fcb     255
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb      47
		fcb     255
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54
		fcb      55
		fcb     255
		fcb      57
		fcb      58
		fcb      59
		fcb      60
		fcb      61
		fcb      62
		fcb      63
		fcb     255

* dir table for moving west
DIRW
		fcb     255
		fcb       0
		fcb       1
		fcb       2
		fcb       3
		fcb       4
		fcb       5
		fcb       6
		fcb     255
		fcb       8
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb     255
		fcb      16
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb     255
		fcb      24
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb     255
		fcb      32
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb     255
		fcb      40
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb     255
		fcb      48
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54
		fcb     255
		fcb      56
		fcb      57
		fcb      58
		fcb      59
		fcb      60
		fcb      61
		fcb      62

* dir table for moving north east
DIRNE
MIDDIR
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb      15
		fcb     255
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb      23
		fcb     255
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb      31
		fcb     255
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb      39
		fcb     255
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb      47
		fcb     255
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54
		fcb      55
		fcb     255
		fcb      57
		fcb      58
		fcb      59
		fcb      60
		fcb      61
		fcb      62
		fcb      63
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255

* dir table for moving north west
DIRNW
		fcb     255
		fcb       8
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb     255
		fcb      16
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb     255
		fcb      24
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb     255
		fcb      32
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb     255
		fcb      40
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb     255
		fcb      48
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54
		fcb     255
		fcb      56
		fcb      57
		fcb      58
		fcb      59
		fcb      60
		fcb      61
		fcb      62
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255

* dir table for moving south east
DIRSE
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb       1
		fcb       2
		fcb       3
		fcb       4
		fcb       5
		fcb       6
		fcb       7
		fcb     255
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb      15
		fcb     255
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb      23
		fcb     255
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb      31
		fcb     255
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb      39
		fcb     255
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb      47
		fcb     255
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54
		fcb      55
		fcb     255

* dir table for moving south west
DIRSW
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb       0
		fcb       1
		fcb       2
		fcb       3
		fcb       4
		fcb       5
		fcb       6
		fcb     255
		fcb       8
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb     255
		fcb      16
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb     255
		fcb      24
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb     255
		fcb      32
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb     255
		fcb      40
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb     255
		fcb      48
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54


* dir table for knight 0
NDIR0
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb      23
		fcb     255
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb      31
		fcb     255
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb      39
		fcb     255
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb      47
		fcb     255
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54
		fcb      55
		fcb     255
		fcb      57
		fcb      58
		fcb      59
		fcb      60
		fcb      61
		fcb      62
		fcb      63
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255

* dir table for knight 1
NDIR1
		fcb     255
		fcb      16
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb     255
		fcb      24
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb     255
		fcb      32
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb     255
		fcb      40
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb     255
		fcb      48
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54
		fcb     255
		fcb      56
		fcb      57
		fcb      58
		fcb      59
		fcb      60
		fcb      61
		fcb      62
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255

* dir table for knight 2
NDIR2
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb      15
		fcb     255
		fcb     255
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb      23
		fcb     255
		fcb     255
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb      31
		fcb     255
		fcb     255
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb      39
		fcb     255
		fcb     255
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb      47
		fcb     255
		fcb     255
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54
		fcb      55
		fcb     255
		fcb     255
		fcb      58
		fcb      59
		fcb      60
		fcb      61
		fcb      62
		fcb      63
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255

* dir table for knight 3
NDIR3
		fcb     255
		fcb     255
		fcb       8
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb     255
		fcb     255
		fcb      16
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb     255
		fcb     255
		fcb      24
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb     255
		fcb     255
		fcb      32
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb     255
		fcb     255
		fcb      40
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb     255
		fcb     255
		fcb      48
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb     255
		fcb     255
		fcb      56
		fcb      57
		fcb      58
		fcb      59
		fcb      60
		fcb      61
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255

* dir table for knight 4
NDIR4
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb       1
		fcb       2
		fcb       3
		fcb       4
		fcb       5
		fcb       6
		fcb       7
		fcb     255
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb      15
		fcb     255
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb      23
		fcb     255
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb      31
		fcb     255
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb      39
		fcb     255
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb      47
		fcb     255

* dir table for knight 5
NDIR5
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb       0
		fcb       1
		fcb       2
		fcb       3
		fcb       4
		fcb       5
		fcb       6
		fcb     255
		fcb       8
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb     255
		fcb      16
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb     255
		fcb      24
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb     255
		fcb      32
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb     255
		fcb      40
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46

* dir table for knight 6
NDIR6
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb       2
		fcb       3
		fcb       4
		fcb       5
		fcb       6
		fcb       7
		fcb     255
		fcb     255
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb      14
		fcb      15
		fcb     255
		fcb     255
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb      22
		fcb      23
		fcb     255
		fcb     255
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb      30
		fcb      31
		fcb     255
		fcb     255
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb      38
		fcb      39
		fcb     255
		fcb     255
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb      46
		fcb      47
		fcb     255
		fcb     255
		fcb      50
		fcb      51
		fcb      52
		fcb      53
		fcb      54
		fcb      55
		fcb     255
		fcb     255

* dir table for knight 7
NDIR7
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb     255
		fcb       0
		fcb       1
		fcb       2
		fcb       3
		fcb       4
		fcb       5
		fcb     255
		fcb     255
		fcb       8
		fcb       9
		fcb      10
		fcb      11
		fcb      12
		fcb      13
		fcb     255
		fcb     255
		fcb      16
		fcb      17
		fcb      18
		fcb      19
		fcb      20
		fcb      21
		fcb     255
		fcb     255
		fcb      24
		fcb      25
		fcb      26
		fcb      27
		fcb      28
		fcb      29
		fcb     255
		fcb     255
		fcb      32
		fcb      33
		fcb      34
		fcb      35
		fcb      36
		fcb      37
		fcb     255
		fcb     255
		fcb      40
		fcb      41
		fcb      42
		fcb      43
		fcb      44
		fcb      45
		fcb     255
		fcb     255
		fcb      48
		fcb      49
		fcb      50
		fcb      51
		fcb      52
		fcb      53

* piece values for center control
PVO		fcb	0		0
		fcb	1		1 C_pawn
		fcb	0		2
		fcb	4		3 C_knight
		fcb	3		4 C_bishop
		fcb	2		5 C_rook
		fcb	-1		6 C_king
		fcb	0		7
		fcb	0		8
		fcb	1		9 C_queen

* center control table
BV
		fcb	0
		fcb	1
		fcb	2
		fcb	3
		fcb	3
		fcb	2
		fcb	1
		fcb	0
		fcb	1
		fcb	3
		fcb	4
		fcb	5
		fcb	5
		fcb	4
		fcb	3
		fcb	1
		fcb	2
		fcb	4
		fcb	6
		fcb	7
		fcb	7
		fcb	6
		fcb	4
		fcb	2
		fcb	3
		fcb	5
		fcb	7
		fcb	8
		fcb	8
		fcb	7
		fcb	5
		fcb	3
		fcb	3
		fcb	5
		fcb	7
		fcb	8
		fcb	8
		fcb	7
		fcb	5
		fcb	3
		fcb	2
		fcb	4
		fcb	6
		fcb	7
		fcb	7
		fcb	6
		fcb	4
		fcb	2
		fcb	1
		fcb	3
		fcb	4
		fcb	5
		fcb	5
		fcb	4
		fcb	3
		fcb	1
		fcb	0
		fcb	1
		fcb	2
		fcb	3
		fcb	3
		fcb	2
		fcb	1
		fcb	0

*****************************************************************************
*                                                                           *
*  These are the routines to check if a particular square is under attack.  *
*                                                                           *
* X points to the square to check                                           *
* returns Z set if attacked, cleared if not                                 *
* X is preserved if not attacked, but A is never preserved                  *
*                                                                           *
*****************************************************************************
C_attacked	txa			save the location we're checking
		ldx	DIRNE,x		get the location to check for
		bmi	Ca_pawn		if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_pawn		if it's not an opponents pawn
		bne	Ca_pawn		keep checking
		rts			otherwise return with Z set
Ca_pawn		tax			restore the location we're checking
		ldx	DIRNW,x		get next location to check for
		bmi	Ca_knight_1	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_pawn		if it's not an opponents pawn
		bne	Ca_knight_1	keep checking
		rts
Ca_knight_1	tax
		ldx	NDIR0,x		get knight location to check for
		bmi	Ca_knight_2	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_knight	if it's not an opponents knight
		bne	Ca_knight_2	keep checking
		rts
Ca_knight_2	tax
		ldx	NDIR1,x		get knight location to check for
		bmi	Ca_knight_3	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_knight	if it's not an opponents knight
		bne	Ca_knight_3	keep checking
		rts
Ca_knight_3	tax
		ldx	NDIR2,x		get knight location to check for
		bmi	Ca_knight_4	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_knight	if it's not an opponents knight
		bne	Ca_knight_4	keep checking
		rts
Ca_knight_4	tax
		ldx	NDIR3,x		get knight location to check for
		bmi	Ca_knight_5	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_knight	if it's not an opponents knight
		bne	Ca_knight_5	keep checking
		rts
Ca_knight_5	tax
		ldx	NDIR4,x		get knight location to check for
		bmi	Ca_knight_6	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_knight	if it's not an opponents knight
		bne	Ca_knight_6	keep checking
		rts
Ca_knight_6	tax
		ldx	NDIR5,x		get knight location to check for
		bmi	Ca_knight_7	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_knight	if it's not an opponents knight
		bne	Ca_knight_7	keep checking
		rts
Ca_knight_7	tax
		ldx	NDIR6,x		get knight location to check for
		bmi	Ca_knight_8	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_knight	if it's not an opponents knight
		bne	Ca_knight_8	keep checking
		rts
Ca_knight_8	tax
		ldx	NDIR7,x		get last knight location to check for
		bmi	Ca_king_1	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_knight	if it's not an opponents knight
		bne	Ca_king_1	keep checking
		rts
Ca_king_1	tax
		ldx	DIRN,x		get king location to check for
		bmi	Ca_king_2	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_king		if it's not an opponents king
		bne	Ca_king_2	keep checking
		rts
Ca_king_2	tax
		ldx	DIRS,x		get king location to check for
		bmi	Ca_king_3	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_king		if it's not an opponents king
		bne	Ca_king_3	keep checking
		rts
Ca_king_3	tax
		ldx	DIRE,x		get king location to check for
		bmi	Ca_king_4	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_king		if it's not an opponents king
		bne	Ca_king_4	keep checking
		rts
Ca_king_4	tax
		ldx	DIRW,x		get king location to check for
		bmi	Ca_king_5	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_king		if it's not an opponents king
		bne	Ca_king_5	keep checking
		rts
Ca_king_5	tax
		ldx	DIRNE,x		get king location to check for
		bmi	Ca_king_6	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_king		if it's not an opponents king
		bne	Ca_king_6	keep checking
		rts
Ca_king_6	tax
		ldx	DIRNW,x		get king location to check for
		bmi	Ca_king_7	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_king		if it's not an opponents king
		bne	Ca_king_7	keep checking
		rts
Ca_king_7	tax
		ldx	DIRSE,x		get king location to check for
		bmi	Ca_king_8	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_king		if it's not an opponents king
		bne	Ca_king_8	keep checking
		rts
Ca_king_8	tax
		ldx	DIRSW,x		get last king location to check for
		bmi	Ca_other_1	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#O_king		if it's not an opponents king
		bne	Ca_other_1	keep checking
		rts

* now check for threats from the queen or the rook or bishop
Ca_other_1	tax
Ca_next_1	ldx	DIRN,x		get the piece location to check for
		bmi	Ca_other_2	if off board, check next location
		tst	b,x		if no piece there,
		beq	Ca_next_1	keep looking
		ldx	b,x		get the piece there
		cpx	#O_rook		if it's not an opponents rook
		bne	Ca_check_q_1	see if it's a queen
		rts			return because it's a rook
Ca_check_q_1	cpx	#O_queen	if it's not an opponents queen
		bne	Ca_other_2	keep checking
		rts
Ca_other_2	tax
Ca_next_2	ldx	DIRS,x		get the piece location to check for
		bmi	Ca_other_3	if off board, check next location
		tst	b,x		if no piece there,
		beq	Ca_next_2	keep looking
		ldx	b,x		get the piece there
		cpx	#O_rook		if it's not an opponents rook
		bne	Ca_check_q_2	see if it's a queen
		rts			return because it's a rook
Ca_check_q_2	cpx	#O_queen	if it's not an opponents queen
		bne	Ca_other_3	keep checking
		rts
Ca_other_3	tax
Ca_next_3	ldx	DIRE,x		get the piece location to check for
		bmi	Ca_other_4	if off board, check next location
		tst	b,x		if no piece there,
		beq	Ca_next_3	keep looking
		ldx	b,x		get the piece there
		cpx	#O_rook		if it's not an opponents rook
		bne	Ca_check_q_3	see if it's a queen
		rts			return because it's a rook
Ca_check_q_3	cpx	#O_queen	if it's not an opponents queen
		bne	Ca_other_4	keep checking
		rts
Ca_other_4	tax
Ca_next_4	ldx	DIRW,x		get the piece location to check for
		bmi	Ca_other_5	if off board, check next location
		tst	b,x		if no piece there,
		beq	Ca_next_4	keep looking
		ldx	b,x		get the piece there
		cpx	#O_rook		if it's not an opponents rook
		bne	Ca_check_q_4	see if it's a queen
		rts			return because it's a rook
Ca_check_q_4	cpx	#O_queen	if it's not an opponents queen
		bne	Ca_other_5	keep checking
		rts
Ca_other_5	tax
Ca_next_5	ldx	DIRNE,x		get the piece location to check for
		bmi	Ca_other_6	if off board, check next location
		tst	b,x		if no piece there,
		beq	Ca_next_5	keep looking
		ldx	b,x		get the piece there
		cpx	#O_bishop	if it's not an opponents bishop
		bne	Ca_check_q_5	see if it's a queen
		rts			return because it's a bishop
Ca_check_q_5	cpx	#O_queen	if it's not an opponents queen
		bne	Ca_other_6	keep checking
		rts
Ca_other_6	tax
Ca_next_6	ldx	DIRNW,x		get the piece location to check for
		bmi	Ca_other_7	if off board, check next location
		tst	b,x		if no piece there,
		beq	Ca_next_6	keep looking
		ldx	b,x		get the piece there
		cpx	#O_bishop	if it's not an opponents bishop
		bne	Ca_check_q_6	see if it's a queen
		rts			return because it's a bishop
Ca_check_q_6	cpx	#O_queen	if it's not an opponents queen
		bne	Ca_other_7	keep checking
		rts
Ca_other_7	tax
Ca_next_7	ldx	DIRSE,x		get the piece location to check for
		bmi	Ca_other_8	if off board, check next location
		tst	b,x		if no piece there,
		beq	Ca_next_7	keep looking
		ldx	b,x		get the piece there
		cpx	#O_bishop	if it's not an opponents bishop
		bne	Ca_check_q_7	see if it's a queen
		rts			return because it's a bishop
Ca_check_q_7	cpx	#O_queen	if it's not an opponents queen
		bne	Ca_other_8	keep checking
		rts
Ca_other_8	tax
Ca_next_8	ldx	DIRSW,x		get the piece location to check for
		bmi	Ca_return	if off board, return with Z=0
		tst	b,x		if no piece there,
		beq	Ca_next_8	keep looking
		ldx	b,x		get the piece there
		cpx	#O_bishop	if it's not an opponents bishop
		bne	Ca_check_q_8	see if it's a queen
		rts			return because it's a bishop
Ca_check_q_8	cpx	#O_queen	if it's not an opponents queen
Ca_return	rts			if queen then Z=1, else Z=0

*****************************************************************************
*                                                                           *
* Do the same for opponent                                                  *
*                                                                           *
*****************************************************************************
O_attacked	txa			save the location we're checking
		ldx	DIRNE,x		get the location to check for
		bmi	Oa_pawn		if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_pawn		if it's not a computers pawn
		bne	Oa_pawn		keep checking
		rts			otherwise return with Z set
Oa_pawn		tax			restore the location we're checking
		ldx	DIRNW,x		get next location to check for
		bmi	Oa_knight_1	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_pawn		if it's not a computers pawn
		bne	Oa_knight_1	keep checking
		rts
Oa_knight_1	tax
		ldx	NDIR0,x		get knight location to check for
		bmi	Oa_knight_2	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_knight	if it's not a computers knight
		bne	Oa_knight_2	keep checking
		rts
Oa_knight_2	tax
		ldx	NDIR1,x		get knight location to check for
		bmi	Oa_knight_3	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_knight	if it's not a computers knight
		bne	Oa_knight_3	keep checking
		rts
Oa_knight_3	tax
		ldx	NDIR2,x		get knight location to check for
		bmi	Oa_knight_4	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_knight	if it's not a computers knight
		bne	Oa_knight_4	keep checking
		rts
Oa_knight_4	tax
		ldx	NDIR3,x		get knight location to check for
		bmi	Oa_knight_5	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_knight	if it's not a computers knight
		bne	Oa_knight_5	keep checking
		rts
Oa_knight_5	tax
		ldx	NDIR4,x		get knight location to check for
		bmi	Oa_knight_6	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_knight	if it's not a computers knight
		bne	Oa_knight_6	keep checking
		rts
Oa_knight_6	tax
		ldx	NDIR5,x		get knight location to check for
		bmi	Oa_knight_7	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_knight	if it's not a computers knight
		bne	Oa_knight_7	keep checking
		rts
Oa_knight_7	tax
		ldx	NDIR6,x		get knight location to check for
		bmi	Oa_knight_8	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_knight	if it's not a computers knight
		bne	Oa_knight_8	keep checking
		rts
Oa_knight_8	tax
		ldx	NDIR7,x		get last knight location to check for
		bmi	Oa_king_1	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_knight	if it's not a computers knight
		bne	Oa_king_1	keep checking
		rts
Oa_king_1	tax
		ldx	DIRN,x		get king location to check for
		bmi	Oa_king_2	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_king		if it's not a computers king
		bne	Oa_king_2	keep checking
		rts
Oa_king_2	tax
		ldx	DIRS,x		get king location to check for
		bmi	Oa_king_3	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_king		if it's not a computers king
		bne	Oa_king_3	keep checking
		rts
Oa_king_3	tax
		ldx	DIRE,x		get king location to check for
		bmi	Oa_king_4	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_king		if it's not a computers king
		bne	Oa_king_4	keep checking
		rts
Oa_king_4	tax
		ldx	DIRW,x		get king location to check for
		bmi	Oa_king_5	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_king		if it's not a computers king
		bne	Oa_king_5	keep checking
		rts
Oa_king_5	tax
		ldx	DIRNE,x		get king location to check for
		bmi	Oa_king_6	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_king		if it's not a computers king
		bne	Oa_king_6	keep checking
		rts
Oa_king_6	tax
		ldx	DIRNW,x		get king location to check for
		bmi	Oa_king_7	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_king		if it's not a computers king
		bne	Oa_king_7	keep checking
		rts
Oa_king_7	tax
		ldx	DIRSE,x		get king location to check for
		bmi	Oa_king_8	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_king		if it's not a computers king
		bne	Oa_king_8	keep checking
		rts
Oa_king_8	tax
		ldx	DIRSW,x		get last king location to check for
		bmi	Oa_other_1	if off board, check next location
		ldx	b,x		get the piece there
		cpx	#C_king		if it's not a computers king
		bne	Oa_other_1	keep checking
		rts

* now check for threats from the queen or the rook
Oa_other_1	tax
Oa_next_1	ldx	DIRN,x		get the piece location to check for
		bmi	Oa_other_2	if off board, check next location
		tst	b,x		if no piece there,
		beq	Oa_next_1	keep looking
		ldx	b,x		get the piece there
		cpx	#C_rook		if it's not a computers rook
		bne	Oa_check_q_1	see if it's a queen
		rts			return because it's a rook
Oa_check_q_1	cpx	#C_queen	if it's not a computers queen
		bne	Oa_other_2	keep checking
		rts
Oa_other_2	tax
Oa_next_2	ldx	DIRS,x		get the piece location to check for
		bmi	Oa_other_3	if off board, check next location
		tst	b,x		if no piece there,
		beq	Oa_next_2	keep looking
		ldx	b,x		get the piece there
		cpx	#C_rook		if it's not a computers rook
		bne	Oa_check_q_2	see if it's a queen
		rts			return because it's a rook
Oa_check_q_2	cpx	#C_queen	if it's not a computers queen
		bne	Oa_other_3	keep checking
		rts
Oa_other_3	tax
Oa_next_3	ldx	DIRE,x		get the piece location to check for
		bmi	Oa_other_4	if off board, check next location
		tst	b,x		if no piece there,
		beq	Oa_next_3	keep looking
		ldx	b,x		get the piece there
		cpx	#C_rook		if it's not a computers rook
		bne	Oa_check_q_3	see if it's a queen
		rts			return because it's a rook
Oa_check_q_3	cpx	#C_queen	if it's not a computers queen
		bne	Oa_other_4	keep checking
		rts
Oa_other_4	tax
Oa_next_4	ldx	DIRW,x		get the piece location to check for
		bmi	Oa_other_5	if off board, check next location
		tst	b,x		if no piece there,
		beq	Oa_next_4	keep looking
		ldx	b,x		get the piece there
		cpx	#C_rook		if it's not a computers rook
		bne	Oa_check_q_4	see if it's a queen
		rts			return because it's a rook
Oa_check_q_4	cpx	#C_queen	if it's not a computers queen
		bne	Oa_other_5	keep checking
		rts
Oa_other_5	tax
Oa_next_5	ldx	DIRNE,x		get the piece location to check for
		bmi	Oa_other_6	if off board, check next location
		tst	b,x		if no piece there,
		beq	Oa_next_5	keep looking
		ldx	b,x		get the piece there
		cpx	#C_bishop	if it's not a computers bishop
		bne	Oa_check_q_5	see if it's a queen
		rts			return because it's a bishop
Oa_check_q_5	cpx	#C_queen	if it's not a computers queen
		bne	Oa_other_6	keep checking
		rts
Oa_other_6	tax
Oa_next_6	ldx	DIRNW,x		get the piece location to check for
		bmi	Oa_other_7	if off board, check next location
		tst	b,x		if no piece there,
		beq	Oa_next_6	keep looking
		ldx	b,x		get the piece there
		cpx	#C_bishop	if it's not a computers bishop
		bne	Oa_check_q_6	see if it's a queen
		rts			return because it's a bishop
Oa_check_q_6	cpx	#C_queen	if it's not a computers queen
		bne	Oa_other_7	keep checking
		rts
Oa_other_7	tax
Oa_next_7	ldx	DIRSE,x		get the piece location to check for
		bmi	Oa_other_8	if off board, check next location
		tst	b,x		if no piece there,
		beq	Oa_next_7	keep looking
		ldx	b,x		get the piece there
		cpx	#C_bishop	if it's not a computers bishop
		bne	Oa_check_q_7	see if it's a queen
		rts			return because it's a bishop
Oa_check_q_7	cpx	#C_queen	if it's not a computers queen
		bne	Oa_other_8	keep checking
		rts
Oa_other_8	tax
Oa_next_8	ldx	DIRSW,x		get the piece location to check for
		bmi	Oa_return	if off board, return with Z=0
		tst	b,x		if no piece there,
		beq	Oa_next_8	keep looking
		ldx	b,x		get the piece there
		cpx	#C_bishop	if it's not a computers bishop
		bne	Oa_check_q_8	see if it's a queen
		rts			return because it's a bishop
Oa_check_q_8	cpx	#C_queen	if it's not a computers queen
Oa_return	rts			if queen then Z=1, else Z=0

*****************************************************************************
*****************************************************************************
*                                                                           *
* The Move Generator                                                        *
*                                                                           *
*****************************************************************************
*****************************************************************************

*****************************************************************************
*                                                                           *
* Generate the next capture move for the computer                           *
*                                                                           *
*****************************************************************************
cC_vector	jmp	cCpdir0		pawn capture direction 1
		jmp	cCpdir1		pawn capture direction 2
		jmp	cCpdir2		pawn promotion move
		jmp	next_cC_pos	end of pawn moves
cCvb		jmp	cCbdir0		bishop capture move
		jmp	cCbdir1		bishop capture move
		jmp	cCbdir2		bishop capture move
		jmp	cCbdir3		bishop capture move
		jmp	next_cC_pos	end of bishop moves
cCvr		jmp	cCrdir0		rook capture move
		jmp	cCrdir1		rook capture move
		jmp	cCrdir2		rook capture move
		jmp	cCrdir3		rook capture move
		jmp	next_cC_pos	end of rook moves
cCvq		jmp	cCqdir0		queen capture move
		jmp	cCqdir1		queen capture move
		jmp	cCqdir2		queen capture move
		jmp	cCqdir3		queen capture move
		jmp	cCqdir4		queen capture move
		jmp	cCqdir5		queen capture move
		jmp	cCqdir6		queen capture move
		jmp	cCqdir7		queen capture move
		jmp	next_cC_pos	end of queen moves
cCvkn		jmp	cCkndir0	knight capture move
		jmp	cCkndir1	knight capture move
		jmp	cCkndir2	knight capture move
		jmp	cCkndir3	knight capture move
		jmp	cCkndir4	knight capture move
		jmp	cCkndir5	knight capture move
		jmp	cCkndir6	knight capture move
		jmp	cCkndir7	knight capture move
		jmp	next_cC_pos	end of knight moves
cCvk		jmp	cCkdir0		king capture move
		jmp	cCkdir1		king capture move
		jmp	cCkdir2		king capture move
		jmp	cCkdir3		king capture move
		jmp	cCkdir4		king capture move
		jmp	cCkdir5		king capture move
		jmp	cCkdir6		king capture move
		jmp	cCkdir7		king capture move
		jmp	next_cC_pos	end of king moves
cC_offset	fcb	0		not used
		fcb	0		offset for pawn
		fcb	0		not used
		fcb	cCvkn-cC_vector offset for knight
		fcb	cCvb-cC_vector	offset for bishop
		fcb	cCvr-cC_vector	offset for rook
		fcb	cCvk-cC_vector	offset for king
		fcb	0		not used
		fcb	0		not used
		fcb	cCvq-cC_vector	offset for queen

*****************************************************************************
*                                                                           *
* Entry point of routine to generate the next move for the computer         *
*                                                                           *
*****************************************************************************
next_C_move	ldx	l		get the stack pointer
		lda	scap,x		are we considering capture moves?
		bne	get_next_cC	yes, do the next capture move
		jmp	get_next_nC	otherwise get next non capture move

*****************************************************************************
*                                                                           *
* Entry point of routine to generate the next capture move for the computer *
*                                                                           *
*****************************************************************************
get_next_cC	lda	sdir,x  	get stacked dir, frm and
		sta	dir     	save them in temporary
		lda	sfrm,x  	locations for convenient
		sta	frm     	access
cC_new_move	clr	mf		set move flag to normal
		ldx	frm     	index FROM position
		bra	next_cC_get	go get the piece to move
next_cC_loop	decx			point to next board position
		bmi	cC_exit		if reached end of board, exit
next_cC_get	lda	b,x		0 or plus, it's blank or computer
		beq	next_cC_loop	a blank square!
		bmi	next_cC_loop	only move computers pieces
		stx	frm		save FROM position
		tax			index the piece
		sta	temp		save the piece
		lda	dir		multiply direction by 3
		lsla			first multiply by 2
		add	dir		result is dir multiplied by 3
		add	cC_offset,x 	add piece offset specified in the table
		tax			index it
		jmp	cC_vector,x	jump based on dir*3+taboffset[piece]

cC_exit		ldx	l		get the stack pointer
		clr	scap,x		non capture moves from now on
		clr	dir		clear move direction
		lda	#63		init FROM to first position
		sta	frm
		sta	pto		init TO to first position
		jmp	nC_new_move	get first non capture move

next_cC_pos	clr	dir		reset direction for next position
		ldx	frm		index FROM position
		bra	next_cC_loop	process the new position
*
* generate the pawn moves
*
cCpdir0		inc	dir		(next direction)
		ldx	frm		determine position to move to
		ldx	DIRNE,x		ie north east
		bmi	cCpdir1		off the board, try next position
		lda	b,x		get what's on the board
		bmi	cC_prom		if opponent, this is a capture move
		bne	cCpdir1		if not blank, try next direction
		cpx	#40		check destination to 6rd rank for ep
		blo	cCpdir1		lower, try other direction
		cpx	#48             between 40 and 47 is 6th rank
		bhs	cCpdir1		not between, try other direction
		stx	pto		save destination
		txa			put in A for manipulation
		sub	#8		look for opponent pawn here
		tax			index the location
		ldx	b,x		get the piece
		cpx	#O_pawn		an opponent pawn?
		bne	cCpdir1		no, try the other capture direction
		ldx	l		point to stack
		cmp	spto_1,x	was last move this pawn?
		bne	cCpdir1		destination wasn't try other direction
		add	#16		check origin for 2 square pawn move
		cmp	sfrm_1,x	same as origin for last move
		bne	cCpdir1		no, not ep, try other direction
		ldx	#ep		indicate ep move
		stx	mf
		ldx	pto		get destination
		jmp	return_cC_p	return with capture move
*
* check for promotion, if a pawn moves to 8th rank it becomes a queen
* so there is no need to check for upper bound of 64
*
cC_prom		cpx	#56		capture and promotion?
		bhs	cC_cap_prom	yes indicate promotion
		jmp	return_cC_x	exit move in x
cCpdir2		inc	dir		(next direction)
		lda     frm		get from position
		add	#8		move north
		cmp	#56      	if 8th rank ie >=56 check for blank
		blo	next_cC_pos	otherwise check next position
		tax             	put in X for checking occupance
		lda	b,x		set flags for checking occupancy
		bne	next_cC_pos	not vacant, next position
cC_cap_prom	lda	#promotion	indicate promotion
		sta	mf
		jmp	return_cC_x	exit move in x
*
* try other direction for capture
*
cCpdir1		inc	dir		(next direction)
		ldx	frm		determine position to move to
		ldx	DIRNW,x		ie north west
		bmi	cCpdir2		off the board, try next position
		lda	b,x		get what's on the board
		bmi	cC_prom		if opponent, this is a capture move
		bne	cCpdir2		if not blank, try next direction
		cpx	#40		check destination to 6rd rank for ep
		blo	cCpdir2		lower, try other direction
		cpx	#48             between 40 and 47 is 6th rank
		bhs	cCpdir2		not between, try other direction
		stx	pto		save destination
		txa			put in A for manipulation
		sub	#8		look for opponent pawn here
		tax			index the location
		ldx	b,x		get the piece
		cpx	#O_pawn		an opponent pawn?
		bne	cCpdir2		no, try the other capture direction
		ldx	l		point to stack
		cmp	spto_1,x	was last move this pawn?
		bne	cCpdir2		destination wasn't try other direction
		add	#16		check origin for 2 square pawn move
		cmp	sfrm_1,x	same as origin for last move
		bne	cCpdir2		no, not ep, try other direction
		ldx	#ep		indicate ep move
		stx	mf
		ldx	pto		get destination
		jmp	return_cC_p	return with capture move
*
* generate bishop moves
*
cCbdir0		inc	dir		ready for next direction
		ldx	frm		index from position
cCbtrynext_0	ldx	DIRNE,x		get next board position
		bmi	cCbdir1		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCbtrynext_0	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCbdir1		inc	dir		ready for next direction
		ldx	frm		index from position
cCbtrynext_1	ldx	DIRNW,x		get next board position
		bmi	cCbdir2		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCbtrynext_1	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCbdir2		inc	dir		ready for next direction
		ldx	frm		index from position
cCbtrynext_2	ldx	DIRSE,x		get next board position
		bmi	cCbdir3		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCbtrynext_2	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCbdir3		inc	dir		ready for next direction
		ldx	frm		index from position
cCbtrynext_3	ldx	DIRSW,x		get next board position
		bmi	cCb_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	cCbtrynext_3	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCb_pos		jmp	next_cC_pos	try the next board position
*
* generate rook moves
*
cCrdir0		inc	dir		ready for next direction
		ldx	frm		index from position
cCrtrynext_0	ldx	DIRN,x		get next board position
		bmi	cCrdir1		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCrtrynext_0	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCrdir1		inc	dir		ready for next direction
		ldx	frm		index from position
cCrtrynext_1	ldx	DIRS,x		get next board position
		bmi	cCrdir2		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCrtrynext_1	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCrdir2		inc	dir		ready for next direction
		ldx	frm		index from position
cCrtrynext_2	ldx	DIRE,x		get next board position
		bmi	cCrdir3		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCrtrynext_2	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCrdir3		inc	dir		ready for next direction
		ldx	frm		index from position
cCrtrynext_3	ldx	DIRW,x		get next board position
		bmi	cCr_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	cCrtrynext_3	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCr_pos		jmp	next_cC_pos	try the next board position
*
* link to return_cC_x
*
cC_x_link	jmp	return_cC_x	exit move in x
*
* generate queens moves
*
cCqdir0		inc	dir		ready for next direction
		ldx	frm		index from position
cCqtrynext_0	ldx	DIRNE,x		get next board position
		bmi	cCqdir1		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCqtrynext_0	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCqdir1		inc	dir		ready for next direction
		ldx	frm		index from position
cCqtrynext_1	ldx	DIRNW,x		get next board position
		bmi	cCqdir2		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCqtrynext_1	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCqdir2		inc	dir		ready for next direction
		ldx	frm		index from position
cCqtrynext_2	ldx	DIRSE,x		get next board position
		bmi	cCqdir3		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCqtrynext_2	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCqdir3		inc	dir		ready for next direction
		ldx	frm		index from position
cCqtrynext_3	ldx	DIRSW,x		get next board position
		bmi	cCqdir4		if off the board, next position
		lda	b,x		see what's on the board here
		beq	cCqtrynext_3	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCqdir4		inc	dir		ready for next direction
		ldx	frm		index from position
cCqtrynext_4	ldx	DIRN,x		get next board position
		bmi	cCqdir5		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCqtrynext_4	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCqdir5		inc	dir		ready for next direction
		ldx	frm		index from position
cCqtrynext_5	ldx	DIRS,x		get next board position
		bmi	cCqdir6		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCqtrynext_5	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCqdir6		inc	dir		ready for next direction
		ldx	frm		index from position
cCqtrynext_6	ldx	DIRE,x		get next board position
		bmi	cCqdir7		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cCqtrynext_6	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCqdir7		inc	dir		ready for next direction
		ldx	frm		index from position
cCqtrynext_7	ldx	DIRW,x		get next board position
		bmi	cCq_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	cCqtrynext_7	if nothing, try next location
		bmi	cC_x_link	if <0 (ie opponent piece) move ok
cCq_pos		jmp	next_cC_pos	try the next board position
*
* another link to return_cC_x
*
cC_x_link2	jmp	return_cC_x	exit move in x
*
* generate knight moves
*
cCkndir0	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR0,x		get next board position
		bmi	cCkndir1	if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	cC_x_link2	if <0 (ie opponent piece) move ok
cCkndir1	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR1,x		get next board position
		bmi	cCkndir2	if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	cC_x_link2	if <0 (ie opponent piece) move ok
cCkndir2	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR2,x		get next board position
		bmi	cCkndir3	if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	cC_x_link2	if <0 (ie opponent piece) move ok
cCkndir3	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR3,x		get next board position
		bmi	cCkndir4	if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	cC_x_link2	if <0 (ie opponent piece) move ok
cCkndir4	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR4,x		get next board position
		bmi	cCkndir5	if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	cC_x_link2	if <0 (ie opponent piece) move ok
cCkndir5	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR5,x		get next board position
		bmi	cCkndir6	if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	cC_x_link2	if <0 (ie opponent piece) move ok
cCkndir6	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR6,x		get next board position
		bmi	cCkndir7	if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	cC_x_link2	if <0 (ie opponent piece) move ok
cCkndir7	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR7,x		get next board position
		bmi	cCkn_pos	if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	cC_x_link2	if <0 (ie opponent piece) move ok
cCkn_pos	jmp	next_cC_pos	try the next board position
*
* generate king moves, essentially the same as above except using
* different direction table values
*
cCkdir0		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRNE,x		get next board position
		bmi	cCkdir1		if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	return_cC_k	if opponent piece, move ok
cCkdir1		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRNW,x		get next board position
		bmi	cCkdir2		if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	return_cC_k	opponent piece, move ok
cCkdir2		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRSE,x		get next board position
		bmi	cCkdir3		if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	return_cC_k	if opponent piece move ok
cCkdir3		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRSW,x		get next board position
		bmi	cCkdir4		if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	return_cC_k	if opponent piece move ok
cCkdir4		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRN,x		get next board position
		bmi	cCkdir5		if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	return_cC_k	if opponent piece move ok
cCkdir5		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRS,x		get next board position
		bmi	cCkdir6		if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	return_cC_k	if opponent piece move ok
cCkdir6		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRE,x		get next board position
		bmi	cCkdir7		if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	return_cC_k	if opponent piece move ok
cCkdir7		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRW,x		get next board position
		bmi	cCk_pos		if off the board, next direction
		lda	b,x		see what's on the board here
		bmi	return_cC_k	if opponent piece move ok
cCk_pos		jmp	next_cC_pos	try the next board position
return_cC_k	stx	Ckp		a capture, save new king position
		txa			save TO position in A
		ldx	l		indicate cannot castle
		clr	sscf,x
		tax			get TO position and fall through
*
* try the move, if not in check return it, otherwise try another
*
return_cC_x     stx	pto		save destination position
return_cC_p	lda	b,x		get the captured piece
		sta	stp		and save it
		ldx	frm		get FROM position
		clr	b,x		remove the piece on the move
		lda	mf		get the move flag
		beq	cC_norm		nothing special, normal capture
		bpl	cC_epas		ep pawn capture
		lda	stp		indicate promotion score by
		sub	#8		subtracting 8 from captured piece
		sta	stp		save it
		lda	#C_queen	replace pawn with queen
		sta	temp
		bra	cC_rest		continue with rest of move
cC_epas		lda	#O_pawn		must be en passan, pawn capture
		sta	stp		save pawn as captured
		lda	pto		get destination position
		sub	#8		adjust for
		tax			position of captured pawn
		clr	b,x		remove captured pawn
cC_norm         lda	temp		get piece on move
cC_rest		ldx	pto		put piece on move at destination
		sta	b,x
*
* check if valid move, ie not left king in check
*
		ldx	Ckp             get kings position
		jsr	C_attacked      see if computer is in check
		bne	cC_moveok	not in check, move ok
*
* king in check, not a valid move, restore board and try
* another move
*
		jsr	restore_cC	restore computers capture move
		jmp	cC_new_move	try another move
*
* a valid move so return it
*
cC_moveok	ldx	l		get search level (stack pointer)
		lda	mf		save move flag
		sta	ssmf,x
		lda     pto		save position to
		sta	spto,x		back on the search stack
		lda	dir		save dir
		sta	sdir,x
		lda	frm		save frm
		sta	sfrm,x
		lda	stp		captured piece
		sta	sstp,x
		clc			indicate a valid capture
		rts

*****************************************************************************
*****************************************************************************
*                                                                           *
* Generate the next non capture move for the computer                       *
*                                                                           *
*****************************************************************************
*****************************************************************************
nC_vector	jmp	nCpdir0		pawn non capture direction 1
		jmp	nCpdir1		pawn non capture direction 2
		jmp	next_nC_pos	end of pawn moves
nCvb		jmp	nCbdir0		bishop non capture move
		jmp	nCbdir1		bishop non capture move
		jmp	nCbdir2		bishop non capture move
		jmp	nCbdir3		bishop non capture move
nCvr		jmp	nCrdir0		rook non capture move
		jmp	nCrdir1		rook non capture move
		jmp	nCrdir2		rook non capture move
		jmp	nCrdir3		rook non capture move
nCvq		jmp	nCqdir0		queen non capture move
		jmp	nCqdir1		queen non capture move
		jmp	nCqdir2		queen non capture move
		jmp	nCqdir3		queen non capture move
		jmp	nCqdir4		queen non capture move
		jmp	nCqdir5		queen non capture move
		jmp	nCqdir6		queen non capture move
		jmp	nCqdir7		queen non capture move
nCvkn		jmp	nCkndir0	knight non capture move
		jmp	nCkndir1	knight non capture move
		jmp	nCkndir2	knight non capture move
		jmp	nCkndir3	knight non capture move
		jmp	nCkndir4	knight non capture move
		jmp	nCkndir5	knight non capture move
		jmp	nCkndir6	knight non capture move
		jmp	nCkndir7	knight non capture move
		jmp	next_nC_pos	end of knight moves
nCvk		jmp	nCkdir0		king non capture move
		jmp	nCkdir1		king non capture move
		jmp	nCkdir2		king non capture move
		jmp	nCkdir3		king non capture move
		jmp	nCkdir4		king non capture move
		jmp	nCkdir5		king non capture move
		jmp	nCkdir6		king non capture move
		jmp	nCkdir7		king non capture move
		jmp	nCcastle1	queen side castle move
		jmp	nCcastle2	king side castle move
		jmp	next_nC_pos	end of king moves
nC_offset	fcb	0		not used
		fcb	0		offset for pawn
		fcb	0		not used
		fcb	nCvkn-nC_vector offset for knight
		fcb	nCvb-nC_vector	offset for bishop
		fcb	nCvr-nC_vector	offset for rook
		fcb	nCvk-nC_vector	offset for king
		fcb	0		not used
		fcb	0		not used
		fcb	nCvq-nC_vector	offset for queen

*****************************************************************************
*                                                                           *
* Entry point of routine to generate the next non capture move for computer *
*                                                                           *
*****************************************************************************
get_next_nC	lda	sdir,x  	dir, frm, and pto and
		sta	dir     	save them in temporary
		lda	sfrm,x  	locations for convenient
		sta	frm     	access
		lda	spto,x
		sta	pto
nC_new_move	clr	mf		set move flag to normal
*
* on entry, process the current position,
* get the current piece pointed to by the current from position pointer
*
		ldx	frm     	get the piece to move
		lda	b,x		0 or plus, it's blank or computer
		beq	next_nC_loop	a blank square!
		bmi	next_nC_loop	only move computers pieces
proc_nC_pos	sta	temp		save piece to update the board later
		tax			index the piece
		lda	dir		multiply direction by 3
		lsla			first multiply by 2
		add	dir		result is dir multiplied by 3
		add	nC_offset,x 	add piece offset specified in the table
		tax			index it
		jmp	nC_vector,x	jump based on dir*3+taboffset[piece]
*
* process the next position, ie
* reset the direction (dir),
* find the next opponent piece,
* update from position pointer (frm) and
* initialise the position to pointer (pto) to frm
*
next_nC_pos	clr	dir		reset direction for next position
		ldx	frm     	index FROM position
next_nC_loop	decx			point to next board position
		bmi	nC_exit		if reached end of board, exit
		lda	b,x		0 or plus, it's blank or computer
		beq	next_nC_loop	a blank square!
		bmi	next_nC_loop	only move computers pieces
		stx	frm		save FROM position
		stx	pto	    	initialise TO position to FROM
		bra	proc_nC_pos	process the new position
nC_exit		sec			return C set, indicating end of moves
		rts
*
* generate the non capture pawn moves
*
nCpdir0		inc	dir		(next direction)
		lda	frm     	get the next location
		add	#8 		to move to, ie N
		cmp	#56		don't include promotions
		bhs	next_nC_pos	exit if pto to 8th rank, ie x>=56
		tax			index the location
		lda	b,x		check destination is blank
		bne	next_nC_pos	exit if not vacant
		jmp	return_nC_x	return valid move in x
nCpdir1		inc	dir		(next direction)
		lda	frm		get location from
		cmp	#16		if on 2nd rank, can advance 2
		bhs	next_nC_pos	otherwise, (>=16) try next piece
		add	#16		advance 2 squares
		tax			transfer to x
		lda	b,x		get what at this location
		bne	next_nC_pos	not blank, try next piece
		jmp	return_nC_x	blank, move ok
*
* generate non capture bishop moves
*
nCbdir0		ldx	pto		index TO position
		ldx	DIRNE,x		get next board position
		bmi	nCbndir1	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk1	if blank, move ok
nCbndir1	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCbdir1		ldx	pto		get TO position
		ldx	DIRNW,x		get next board position
		bmi	nCbndir2	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk1	if blank, move ok
nCbndir2	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCbdir2		ldx	pto		get TO position
		ldx	DIRSE,x		get next board position
		bmi	nCbndir3	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk1	if blank, move ok
nCbndir3	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCbdir3		ldx	pto		get TO position
		ldx	DIRSW,x		get next board position
		bmi	nCb_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk1	if blank, move ok
nCb_pos		jmp	next_nC_pos	try the next board position
*
* link to return_nC_x
*
ret_nC_lnk1	jmp	return_nC_x	blank move ok
*
* generate non capture rook moves
*
nCrdir0		ldx	pto		index TO position
		ldx	DIRN,x		get next board position
		bmi	nCrndir1	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk1	if blank, move ok
nCrndir1	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCrdir1		ldx	pto		get TO position
		ldx	DIRS,x		get next board position
		bmi	nCrndir2	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk1	if blank, move ok
nCrndir2	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCrdir2		ldx	pto		get TO position
		ldx	DIRE,x		get next board position
		bmi	nCrndir3	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk1	if blank, move ok
nCrndir3	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCrdir3		ldx	pto		get TO position
		ldx	DIRW,x		get next board position
		bmi	nCr_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk1	if blank, move ok
nCr_pos		jmp	next_nC_pos	try the next board position
*
* generate non capture queen moves
*
nCqdir0		ldx	pto		index TO position
		ldx	DIRNE,x		get next board position
		bmi	nCqndir1	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCqndir1	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCqdir1		ldx	pto		get TO position
		ldx	DIRNW,x		get next board position
		bmi	nCqndir2	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCqndir2	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCqdir2		ldx	pto		get TO position
		ldx	DIRSE,x		get next board position
		bmi	nCqndir3	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCqndir3	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCqdir3		ldx	pto		get TO position
		ldx	DIRSW,x		get next board position
		bmi	nCqndir4	if off the board, next position
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCqndir4	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCqdir4		ldx	pto		index TO position
		ldx	DIRN,x		get next board position
		bmi	nCqndir5	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCqndir5	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCqdir5		ldx	pto		get TO position
		ldx	DIRS,x		get next board position
		bmi	nCqndir6	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCqndir6	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCqdir6		ldx	pto		get TO position
		ldx	DIRE,x		get next board position
		bmi	nCqndir7	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCqndir7	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nCqdir7		ldx	pto		get TO position
		ldx	DIRW,x		get next board position
		bmi	nCq_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCq_pos		jmp	next_nC_pos	try the next board position
*
* another link to return_nC_x
*
ret_nC_lnk2	jmp	return_nC_x	blank move ok
*
* generate non capture knight moves
*
nCkndir0	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR0,x		get next board position
		bmi	nCkndir1	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCkndir1	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR1,x		get next board position
		bmi	nCkndir2	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCkndir2	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR2,x		get next board position
		bmi	nCkndir3	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCkndir3	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR3,x		get next board position
		bmi	nCkndir4	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCkndir4	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR4,x		get next board position
		bmi	nCkndir5	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCkndir5	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR5,x		get next board position
		bmi	nCkndir6	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCkndir6	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR6,x		get next board position
		bmi	nCkndir7	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCkndir7	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR7,x		get next board position
		bmi	nCkn_pos	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nC_lnk2	if blank, move ok
nCkn_pos	jmp	next_nC_pos	try the next board position
*
* generate non capture king moves,
* essentially the same as above except using
* different direction table values
* dir 0-7 are first 8 directions
* dir 8 and 9 are castle moves
*
nCkdir0		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRNE,x		get next board position
		bmi	nCkdir1		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nC_k	if blank, move ok
nCkdir1		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRNW,x		get next board position
		bmi	nCkdir2		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nC_k	if blank, move ok
nCkdir2		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRSE,x		get next board position
		bmi	nCkdir3		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nC_k	if blank, move ok
nCkdir3		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRSW,x		get next board position
		bmi	nCkdir4		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nC_k	if blank, move ok
nCkdir4		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRN,x		get next board position
		bmi	nCkdir5		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nC_k	if blank, move ok
nCkdir5		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRS,x		get next board position
		bmi	nCkdir6		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nC_k	if blank, move ok
nCkdir6		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRE,x		get next board position
		bmi	nCkdir7		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nC_k	if blank, move ok
nCkdir7		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRW,x		get next board position
		bmi	nCcastle1	if off the board, next direction
		lda	b,x		see what's on the board here
		bne	nCcastle1	if not blank, try first castle move
return_nC_k	stx	Ckp		a capture, save new king position
		txa			save TO position in A
		ldx	l		indicate cannot castle
		clr	sscf,x
		tax			get TO position from A
		bra	return_nC_x	return the move

nCcastle1	inc	dir		ready for other castle move
		ldx	l		get stack pointer
		lda	sscf,x		get castle flag
		beq	nCk_pos		can't castle, next board position
		lda	BPOS0		get the piece at location 0
		cmp	#C_rook		check if it is a rook
		bne	nCcastle2	if not, next direction
		lda	BPOS1		location 2
		bne	nCcastle2	not blank, next direction
		lda	BPOS2		location 3
		bne	nCcastle2	not blank, next direction
		lda	BPOS3		location 4
		bne	nCcastle2	not blank, next direction
		ldx	#3		location 4
		jsr	C_attacked	if this location attacked
		beq	nCcastle2	then next direction
		ldx	#4		location 4
		jsr	C_attacked	if this location attacked
		beq	nCcastle2	then next direction
		lda	#-1		indicate queen side ie -ve
		sta	mf		through move flag
		ldx	#2		ok, after all that, castle is ok
		bra	return_nC_k	return the move
nCcastle2	inc	dir		to avoid an infinite loop
		lda	BPOS7		get the piece there at location 8
		cmp	#C_rook		check if it is a rook
		bne	nCk_pos		if not, next direction
		lda	BPOS6		location 7
		bne	nCk_pos		not blank, next direction
		lda	BPOS5		location 6
		bne	nCk_pos		not blank, next direction
		ldx	#5		location 6
		jsr	C_attacked	if this location attacked
		beq	nCk_pos		then end of king moves
		ldx	#4		location 5
		jsr	C_attacked	if this location attacked
		beq	nCk_pos		end of king moves
		lda	#1		king side move flag +ve
		sta	mf		indicate through move flag
		ldx	#6		ok, after all that, castle is ok
		bra	return_nC_k	return with move in a
nCk_pos		jmp	next_nC_pos	try the next board position
*
* return with move in X
*
return_nC_x	stx	pto		x has pto, so save it
		lda	temp		get the piece on the move
		sta	b,x		put it at its destination
		ldx	frm		get FROM position
		clr	b,x             clear the square there
		lda	mf		test for castle moves
		beq	nC_normal	no castle moves, check move is legal
		bmi	nC_queen_cas	mf is -ve a queen side castle
*
* king side castle, still need to move the rook
*
		lda	#C_rook		the rook
		sta	BPOS5		put rook in its place
		clr	BPOS7		clear original position of rook
		bra	nC_normal	continue, check move is legal
*
* queen side castle, still need to move the rook
*
nC_queen_cas	lda	#C_rook		the rook
		sta	BPOS3		put rook in its place
		clr	BPOS0		clear original position of rook
*
* check if valid move, ie not left king in check
*
nC_normal	ldx	Ckp             get kings position
		jsr	C_attacked      see if computer is in check
		bne	nC_moveok	not in check, move ok
*
* king in check, not a valid move, restore board and try
* another move
*
		jsr	restore_nC	restore computers non capture move
		jmp	nC_new_move	try another move
*
* a valid move so return it
*
nC_moveok       ldx	l		get search level (stack pointer)
		lda	mf		save move flag
		sta	ssmf,x
		lda     pto		save position to
		sta	spto,x		back on the search stack
		lda	dir		save dir
		sta	sdir,x
		lda	frm		save frm
		sta	sfrm,x
		clr	sstp,x		there is no captured piece!
		clc			indicate a valid capture
		rts

*****************************************************************************
*****************************************************************************
*                                                                           *
* Generate the next capture move for the opponent                           *
*                                                                           *
*****************************************************************************
*****************************************************************************
cO_vector	jmp	cOpdir0		pawn capture direction 1
		jmp	cOpdir1		pawn capture direction 2
		jmp	cOpdir2		pawn promotion move
		jmp	next_cO_pos	end of pawn moves
cOvb		jmp	cObdir0		bishop capture move
		jmp	cObdir1		bishop capture move
		jmp	cObdir2		bishop capture move
		jmp	cObdir3		bishop capture move
		jmp	next_cO_pos	end of bishop moves
cOvr		jmp	cOrdir0		rook capture move
		jmp	cOrdir1		rook capture move
		jmp	cOrdir2		rook capture move
		jmp	cOrdir3		rook capture move
		jmp	next_cO_pos	end of rook moves
cOvq		jmp	cOqdir0		queen capture move
		jmp	cOqdir1		queen capture move
		jmp	cOqdir2		queen capture move
		jmp	cOqdir3		queen capture move
		jmp	cOqdir4		queen capture move
		jmp	cOqdir5		queen capture move
		jmp	cOqdir6		queen capture move
		jmp	cOqdir7		queen capture move
		jmp	next_cO_pos	end of queen moves
cOvkn		jmp	cOkndir0	knight capture move
		jmp	cOkndir1	knight capture move
		jmp	cOkndir2	knight capture move
		jmp	cOkndir3	knight capture move
		jmp	cOkndir4	knight capture move
		jmp	cOkndir5	knight capture move
		jmp	cOkndir6	knight capture move
		jmp	cOkndir7	knight capture move
		jmp	next_cO_pos	end of knight moves
cOvk		jmp	cOkdir0		king capture move
		jmp	cOkdir1		king capture move
		jmp	cOkdir2		king capture move
		jmp	cOkdir3		king capture move
		jmp	cOkdir4		king capture move
		jmp	cOkdir5		king capture move
		jmp	cOkdir6		king capture move
		jmp	cOkdir7		king capture move
		jmp	next_cO_pos	end of king moves
cO_offset	fcb	0		not used
		fcb	0		offset for pawn
		fcb	0		not used
		fcb	cOvkn-cO_vector offset for knight
		fcb	cOvb-cO_vector	offset for bishop
		fcb	cOvr-cO_vector	offset for rook
		fcb	cOvk-cO_vector	offset for king
		fcb	0		not used
		fcb	0		not used
		fcb	cOvq-cO_vector	offset for queen

*****************************************************************************
*                                                                           *
* Entry point of routine to generate the next capture move for the opponent *
*                                                                           *
*****************************************************************************
next_O_move	ldx	l		get the stack pointer
		lda	scap,x		are we considering capture moves?
		bne	get_next_cO	yes, do the next capture move
		jmp	get_next_nO	otherwise get next non capture move

*****************************************************************************
*                                                                           *
* Entry point of routine to generate the next capture move for the opponent *
*                                                                           *
*****************************************************************************
get_next_cO	lda	sdir,x  	dir, frm, and
		sta	dir     	save them in temporary
		lda	sfrm,x  	locations for convenient
		sta	frm     	access
cO_new_move	clr	mf		set move flag to normal
		ldx	frm
		bra	next_cO_get	go get the piece to move
next_cO_loop	decx			point to next board position
		bmi	cO_exit		if reached end of board, exit
next_cO_get	lda	b,x		0 or plus, it's blank or computer
		bpl	next_cO_loop	only move opponents pieces
		stx	frm		save FROM position
		sta	temp		save the piece
		nega			make it positive to index into table
		tax                     index piece
		lda	dir		multiply direction by 3
		lsla			first multiply by 2
		add	dir		result is dir multiplied by 3
		add	cO_offset,x 	add piece offset specified in the table
		tax			index it
		jmp	cO_vector,x	jump based on dir*3+taboffset[piece]

cO_exit		ldx	l		get the stack pointer
		clr	scap,x		non capture moves from now on
		clr	dir		clear move direction
		lda	#63		init FROM to first position
		sta	frm
		sta	pto		init TO to first position
		jmp	nO_new_move	get first non capture move

next_cO_pos	clr	dir		reset direction for next position
		ldx	frm		index FROM position
		bra	next_cO_loop	process the new position
*
* generate the pawn moves
*
cOpdir0		inc	dir		ready for next direction
		ldx	frm		determine position to move to
		ldx	DIRSE,x		ie south east
		bmi	cOpdir1		off the board, try next position
		lda	b,x		get what's on the board
		beq	cOtryep0	if blank, check for ep capture
		bpl	cO_prom		if computer, this is a capture move
		bra	cOpdir1		neither, try next direction
cOtryep0	cpx	#16		check destination to 6rd rank for ep
		blo	cOpdir1		lower, try other direction
		cpx	#24             between 16 and 23 is 6th rank
		bhs	cOpdir1		not between, try other direction
		stx	pto		save destination
		txa			put in A for manipulation
		add	#8		look for opponent pawn here
		tax			index the location
		ldx	b,x		get the piece
		cpx	#C_pawn		a computer pawn?
		bne	cOpdir1		no, try the other capture direction
		ldx	l		point to stack
		cmp	spto_1,x	was last move this pawn?
		bne	cOpdir1		destination wasn't try other direction
		sub	#16		check origin for 2 square pawn move
		cmp	sfrm_1,x	same as origin for last move
		bne	cOpdir1		no, not ep, try other direction
		ldx	#ep		indicate ep move
		stx	mf
		ldx	pto		get destination
		jmp	return_cO_p	return with capture move
*
* check for promotion, if a pawn moves to 8th rank it becomes a queen
* so there is no need to check for lower bound of 0
*
cO_prom		cpx	#8		capture and promotion?
		blo	cO_cap_prom	yes indicate promotion
		jmp	return_cO_x	exit move in x
cOpdir2         inc	dir		next direction
		lda     frm		get from position
		sub	#8		move south
		cmp	#8      	if 8th rank ie <=7 check for blank
		bhs	next_cO_pos	otherwise check next position
		tax             	put in X for checking occupance
		lda	b,x		set flags for checking occupancy
		bne	next_cO_pos	not vacant, next position
cO_cap_prom	lda	#promotion	indicate promotion
		sta	mf
		jmp	return_cO_x	exit move in x
*
* try other direction for capture
*
cOpdir1		inc	dir		ready for next direction
		ldx	frm		determine position to move to
		ldx	DIRSW,x		ie south east
		bmi	cOpdir2		off the board, try next position
		lda	b,x		get what's on the board
		beq	cOtryep1	if blank, check for ep capture
		bpl	cO_prom		if computer, this is a capture move
		bra	cOpdir2		neither, try next direction
cOtryep1	cpx	#16		check destination to 6rd rank for ep
		blo	cOpdir2		lower, try other direction
		cpx	#24             between 16 and 23 is 6th rank
		bhs	cOpdir2		not between, try other direction
		stx	pto		save destination
		txa			put in A for manipulation
		add	#8		look for opponent pawn here
		tax			index the location
		ldx	b,x		get the piece
		cpx	#C_pawn		a computer pawn?
		bne	cOpdir2		no, try the other capture direction
		ldx	l		point to stack
		cmp	spto_1,x	was last move this pawn?
		bne	cOpdir2		destination wasn't try other direction
		sub	#16		check origin for 2 square pawn move
		cmp	sfrm_1,x	same as origin for last move
		bne	cOpdir2		no, not ep, try other direction
		ldx	#ep		indicate ep move
		stx	mf
		ldx	pto		get destination
		jmp	return_cO_p	return with capture move
*
* generate bishop moves
*
cObdir0		inc	dir		ready for next direction
		ldx	frm		index from position
cObtrynext_0	ldx	DIRNE,x		get next board position
		bmi	cObdir1		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cObtrynext_0	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cObdir1		inc	dir		ready for next direction
		ldx	frm		index from position
cObtrynext_1	ldx	DIRNW,x		get next board position
		bmi	cObdir2		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cObtrynext_1	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cObdir2		inc	dir		ready for next direction
		ldx	frm		index from position
cObtrynext_2	ldx	DIRSE,x		get next board position
		bmi	cObdir3		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cObtrynext_2	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cObdir3		inc	dir		ready for next direction
		ldx	frm		index from position
cObtrynext_3	ldx	DIRSW,x		get next board position
		bmi	cOb_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	cObtrynext_3	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOb_pos		jmp	next_cO_pos	try the next board position
*
* generate rook moves
*
cOrdir0		inc	dir		ready for next direction
		ldx	frm		index from position
cOrtrynext_0	ldx	DIRN,x		get next board position
		bmi	cOrdir1		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cOrtrynext_0	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOrdir1		inc	dir		ready for next direction
		ldx	frm		index from position
cOrtrynext_1	ldx	DIRS,x		get next board position
		bmi	cOrdir2		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cOrtrynext_1	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOrdir2		inc	dir		ready for next direction
		ldx	frm		index from position
cOrtrynext_2	ldx	DIRE,x		get next board position
		bmi	cOrdir3		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cOrtrynext_2	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOrdir3		inc	dir		ready for next direction
		ldx	frm		index from position
cOrtrynext_3	ldx	DIRW,x		get next board position
		bmi	cOr_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	cOrtrynext_3	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOr_pos		jmp	next_cO_pos	try the next board position
*
* link to return_cO_x
*
cO_x_link	jmp	return_cO_x	exit move in x
*
* generate queens moves
*
cOqdir0		inc	dir		ready for next direction
		ldx	frm		index from position
cOqtrynext_0	ldx	DIRNE,x		get next board position
		bmi	cOqdir1		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cOqtrynext_0	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOqdir1		inc	dir		ready for next direction
		ldx	frm		index from position
cOqtrynext_1	ldx	DIRNW,x		get next board position
		bmi	cOqdir2		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cOqtrynext_1	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOqdir2		inc	dir		ready for next direction
		ldx	frm		index from position
cOqtrynext_2	ldx	DIRSE,x		get next board position
		bmi	cOqdir3		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cOqtrynext_2	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOqdir3		inc	dir		ready for next direction
		ldx	frm		index from position
cOqtrynext_3	ldx	DIRSW,x		get next board position
		bmi	cOqdir4		if off the board, next position
		lda	b,x		see what's on the board here
		beq	cOqtrynext_3	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOqdir4		inc	dir		ready for next direction
		ldx	frm		index from position
cOqtrynext_4	ldx	DIRN,x		get next board position
		bmi	cOqdir5		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cOqtrynext_4	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOqdir5		inc	dir		ready for next direction
		ldx	frm		index from position
cOqtrynext_5	ldx	DIRS,x		get next board position
		bmi	cOqdir6		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cOqtrynext_5	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOqdir6		inc	dir		ready for next direction
		ldx	frm		index from position
cOqtrynext_6	ldx	DIRE,x		get next board position
		bmi	cOqdir7		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	cOqtrynext_6	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOqdir7		inc	dir		ready for next direction
		ldx	frm		index from position
cOqtrynext_7	ldx	DIRW,x		get next board position
		bmi	cOq_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	cOqtrynext_7	if nothing, try next location
		bpl	cO_x_link	if >0 (ie computer piece) move ok
cOq_pos		jmp	next_cO_pos	try the next board position
*
* another link to return_cO_x
*
cO_x_link2	jmp	return_cO_x	exit move in x
*
* generate knight moves
*
cOkndir0	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR0,x		get next board position
		bmi	cOkndir1	if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	cO_x_link2	if >0 (ie computer piece) move ok
cOkndir1	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR1,x		get next board position
		bmi	cOkndir2	if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	cO_x_link2	if >0 (ie computer piece) move ok
cOkndir2	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR2,x		get next board position
		bmi	cOkndir3	if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	cO_x_link2	if >0 (ie computer piece) move ok
cOkndir3	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR3,x		get next board position
		bmi	cOkndir4	if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	cO_x_link2	if >0 (ie computer piece) move ok
cOkndir4	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR4,x		get next board position
		bmi	cOkndir5	if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	cO_x_link2	if >0 (ie computer piece) move ok
cOkndir5	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR5,x		get next board position
		bmi	cOkndir6	if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	cO_x_link2	if >0 (ie computer piece) move ok
cOkndir6	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR6,x		get next board position
		bmi	cOkndir7	if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	cO_x_link2	if >0 (ie computer piece) move ok
cOkndir7	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR7,x		get next board position
		bmi	cOkn_pos	if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	cO_x_link2	if >0 (ie computer piece) move ok
cOkn_pos	jmp	next_cO_pos	try the next board position
*
* generate king moves, essentially the same as above except using
* different direction table values
*
cOkdir0		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRNE,x		get next board position
		bmi	cOkdir1		if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	return_cO_k	if computer piece, move ok
cOkdir1		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRNW,x		get next board position
		bmi	cOkdir2		if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	return_cO_k	computer piece, move ok
cOkdir2		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRSE,x		get next board position
		bmi	cOkdir3		if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	return_cO_k	if computer piece move ok
cOkdir3		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRSW,x		get next board position
		bmi	cOkdir4		if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	return_cO_k	if computer piece move ok
cOkdir4		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRN,x		get next board position
		bmi	cOkdir5		if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	return_cO_k	if computer piece move ok
cOkdir5		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRS,x		get next board position
		bmi	cOkdir6		if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	return_cO_k	if computer piece move ok
cOkdir6		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRE,x		get next board position
		bmi	cOkdir7		if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	return_cO_k	if computer piece move ok
cOkdir7		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRW,x		get next board position
		bmi	cOk_pos		if off the board, next direction
		lda	b,x		see what's on the board here
		deca			if +ve then result is 0 or +ve
		bpl	return_cO_k	if computer piece move ok
cOk_pos		jmp	next_cO_pos	try the next board position
return_cO_k	stx	Okp		a capture, save new king position
		txa			save TO position in A
		ldx	l		indicate cannot castle
		clr	sscf,x
		tax			get TO position and fall through
*
* try the move, if not in check return it, otherwise try another
*
return_cO_x     stx	pto		save destination position
return_cO_p	lda	b,x		get the captured piece
		sta	stp		and save it
		ldx	frm		get FROM position
		clr	b,x             remove the piece on the move
		lda	mf		get the move flag
		beq	cO_norm		nothing special, normal capture
		bpl	cO_epas		ep pawn capture
		lda	stp		indicate promotion score by
		add	#8		adding 8 to captured piece
		sta	stp		save it
		lda	#O_queen	replace pawn with queen
		sta	temp
		bra	cO_rest		continue with rest of move
cO_epas		lda	#C_pawn		must be en passan, pawn capture
		sta	stp		save pawn as captured
		lda	pto		get destination position
		sub	#8		adjust for
		tax			position of captured pawn
		clr	b,x             remove capture pawn
cO_norm         lda	temp		get piece on move
cO_rest		ldx	pto		put piece on move at destination
		sta	b,x
*
* check if valid move, ie not left king in check
*
		ldx	Okp             get kings position
		jsr	O_attacked      see if computer is in check
		bne    	cO_moveok	not in check, move ok
*
* king in check, not a valid move, restore board and try
* another move
*
c_restore	jsr	restore_cO	restore opponents capture move
		jmp	cO_new_move	try another move
*
* a valid move so return it
*
cO_moveok	ldx	l		get search level (stack pointer)
		lda	mf		save move flag
		sta	ssmf,x
		lda     pto		save position to
		sta	spto,x		back on the search stack
		lda	dir		save dir
		sta	sdir,x
		lda	frm		save frm
		sta	sfrm,x
		lda	stp		captured piece
		sta	sstp,x
		clc			indicate a valid capture
		rts

*****************************************************************************
*****************************************************************************
*                                                                           *
* Generate the next non capture move for the opponent                       *
*                                                                           *
*****************************************************************************
*****************************************************************************
nO_vector	jmp	nOpdir0		pawn non capture direction 1
		jmp	nOpdir1		pawn non capture direction 2
		jmp	next_nO_pos	end of pawn moves
nOvb		jmp	nObdir0		bishop non capture move
		jmp	nObdir1		bishop non capture move
		jmp	nObdir2		bishop non capture move
		jmp	nObdir3		bishop non capture move
nOvr		jmp	nOrdir0		rook non capture move
		jmp	nOrdir1		rook non capture move
		jmp	nOrdir2		rook non capture move
		jmp	nOrdir3		rook non capture move
nOvq		jmp	nOqdir0		queen non capture move
		jmp	nOqdir1		queen non capture move
		jmp	nOqdir2		queen non capture move
		jmp	nOqdir3		queen non capture move
		jmp	nOqdir4		queen non capture move
		jmp	nOqdir5		queen non capture move
		jmp	nOqdir6		queen non capture move
		jmp	nOqdir7		queen non capture move
nOvkn		jmp	nOkndir0	knight non capture move
		jmp	nOkndir1	knight non capture move
		jmp	nOkndir2	knight non capture move
		jmp	nOkndir3	knight non capture move
		jmp	nOkndir4	knight non capture move
		jmp	nOkndir5	knight non capture move
		jmp	nOkndir6	knight non capture move
		jmp	nOkndir7	knight non capture move
		jmp	next_nO_pos	end of knight moves
nOvk		jmp	nOkdir0		king non capture move
		jmp	nOkdir1		king non capture move
		jmp	nOkdir2		king non capture move
		jmp	nOkdir3		king non capture move
		jmp	nOkdir4		king non capture move
		jmp	nOkdir5		king non capture move
		jmp	nOkdir6		king non capture move
		jmp	nOkdir7		king non capture move
		jmp	nOcastle1	queen side castle move
		jmp	nOcastle2	king side castle move
		jmp	next_nO_pos	end of king moves
nO_offset	fcb	0		not used
		fcb	0		offset for pawn
		fcb	0		not used
		fcb	nOvkn-nO_vector offset for knight
		fcb	nOvb-nO_vector	offset for bishop
		fcb	nOvr-nO_vector	offset for rook
		fcb	nOvk-nO_vector	offset for king
		fcb	0		not used
		fcb	0		not used
		fcb	nOvq-nO_vector	offset for queen


*****************************************************************************
*                                                                           *
* Entry point of routine to generate the next non capture move for opponent *
*                                                                           *
*****************************************************************************
get_next_nO	lda	sdir,x  	dir, frm, and pto and
		sta	dir     	save them in temporary
		lda	sfrm,x  	locations for convenient
		sta	frm     	access
		lda	spto,x
		sta	pto
nO_new_move	clr	mf		set move flag to normal
*
* on entry, process the current position,
* get the current piece pointed to by the current from position pointer
*
		ldx	frm     	get the piece to move
		lda	b,x		0 or plus, it's blank or computer
		bpl	next_nO_loop	only move opponents pieces ie -ve
proc_nO_pos	sta	temp		save piece to update the board later
		nega                    make it +ve
		tax			index the piece
		lda	dir		multiply direction by 3
		lsla			first multiply by 2
		add	dir		result is dir multiplied by 3
		add	nO_offset,x 	add piece offset specified in the table
		tax			index it
		jmp	nO_vector,x	jump based on dir*3+taboffset[piece]
*
* process the next position, ie
* reset the direction (dir),
* find the next opponent piece,
* update from position pointer (frm) and
* initialise the position to pointer (pto) to frm
*
next_nO_pos	clr	dir		reset direction for next position
		ldx	frm     	index FROM position
next_nO_loop	decx			point to next board position
		bmi	nO_exit		if reached end of board, exit
		lda	b,x		0 or plus, it's blank or computer
		bpl	next_nO_loop	only move opponents pieces ie -ve
		stx	frm		save FROM position
		stx	pto	    	initialise TO position to FROM
		bra	proc_nO_pos	process the new position
nO_exit		sec			return C set, indicating end of moves
		rts
*
* generate the non capture pawn moves
*
nOpdir0		inc	dir		(next direction)
		lda	frm     	get the next location
		add	#-8 		to move to, ie S
		cmp	#8		don't include promotions
		blo	next_nO_pos	exit if pto to 8th rank
		tax			index the location
		lda	b,x		check destination is blank
		bne	next_nO_pos	exit if not vacant
		jmp	return_nO_x	return valid move in x
nOpdir1		inc	dir		(next direction)
		lda	frm		get location from
		cmp	#48		if on 2nd rank, pawn can advance 2
		blo	next_nO_pos	otherwise, try next piece
		sub	#16		advance 2 squares
		tax			transfer to x
		lda	b,x		get what at this location
		bne	next_nO_pos	not blank, try next piece
		jmp	return_nO_x	blank, move ok
*
* generate non capture bishop moves
*
nObdir0		ldx	pto		index TO position
		ldx	DIRNE,x		get next board position
		bmi	nObndir1	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk1	if blank, move ok
nObndir1	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nObdir1		ldx	pto		get TO position
		ldx	DIRNW,x		get next board position
		bmi	nObndir2	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk1	if blank, move ok
nObndir2	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nObdir2		ldx	pto		get TO position
		ldx	DIRSE,x		get next board position
		bmi	nObndir3	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk1	if blank, move ok
nObndir3	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nObdir3		ldx	pto		get TO position
		ldx	DIRSW,x		get next board position
		bmi	nOb_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk1	if blank, move ok
nOb_pos		jmp	next_nO_pos	try the next board position
*
* link to return_nO_x
*
ret_nO_lnk1	jmp	return_nO_x	blank move ok
*
* generate non capture rook moves
*
nOrdir0		ldx	pto		index TO position
		ldx	DIRN,x		get next board position
		bmi	nOrndir1	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk1	if blank, move ok
nOrndir1	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nOrdir1		ldx	pto		get TO position
		ldx	DIRS,x		get next board position
		bmi	nOrndir2	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk1	if blank, move ok
nOrndir2	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nOrdir2		ldx	pto		get TO position
		ldx	DIRE,x		get next board position
		bmi	nOrndir3	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk1	if blank, move ok
nOrndir3	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nOrdir3		ldx	pto		get TO position
		ldx	DIRW,x		get next board position
		bmi	nOr_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk1	if blank, move ok
nOr_pos		jmp	next_nO_pos	try the next board position
*
* generate non capture queen moves
*
nOqdir0		ldx	pto		index TO position
		ldx	DIRNE,x		get next board position
		bmi	nOqndir1	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOqndir1	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nOqdir1		ldx	pto		get TO position
		ldx	DIRNW,x		get next board position
		bmi	nOqndir2	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOqndir2	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nOqdir2		ldx	pto		get TO position
		ldx	DIRSE,x		get next board position
		bmi	nOqndir3	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOqndir3	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nOqdir3		ldx	pto		get TO position
		ldx	DIRSW,x		get next board position
		bmi	nOqndir4	if off the board, next position
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOqndir4	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nOqdir4		ldx	pto		index TO position
		ldx	DIRN,x		get next board position
		bmi	nOqndir5	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOqndir5	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nOqdir5		ldx	pto		get TO position
		ldx	DIRS,x		get next board position
		bmi	nOqndir6	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOqndir6	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nOqdir6		ldx	pto		get TO position
		ldx	DIRE,x		get next board position
		bmi	nOqndir7	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOqndir7	inc	dir		ready for next direction
		ldx	frm		index from position
		stx	pto		init TO position
nOqdir7		ldx	pto		get TO position
		ldx	DIRW,x		get next board position
		bmi	nOq_pos		if off the board, next position
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOq_pos		jmp	next_nO_pos	try the next board position
*
* another link to return_nO_x
*
ret_nO_lnk2	jmp	return_nO_x	blank move ok
*
* generate non capture knight moves
*
nOkndir0	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR0,x		get next board position
		bmi	nOkndir1	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOkndir1	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR1,x		get next board position
		bmi	nOkndir2	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOkndir2	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR2,x		get next board position
		bmi	nOkndir3	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOkndir3	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR3,x		get next board position
		bmi	nOkndir4	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOkndir4	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR4,x		get next board position
		bmi	nOkndir5	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOkndir5	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR5,x		get next board position
		bmi	nOkndir6	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOkndir6	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR6,x		get next board position
		bmi	nOkndir7	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOkndir7	inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	NDIR7,x		get next board position
		bmi	nOkn_pos	if off the board, next direction
		lda	b,x		see what's on the board here
		beq	ret_nO_lnk2	if blank, move ok
nOkn_pos	jmp	next_nO_pos	try the next board position
*
* generate non capture king moves,
* essentially the same as above except using
* different direction table values
* dir 0-7 are first 8 directions
* dir 8 and 9 are castle moves
*
nOkdir0		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRNE,x		get next board position
		bmi	nOkdir1		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nO_k	if blank, move ok
nOkdir1		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRNW,x		get next board position
		bmi	nOkdir2		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nO_k	if blank, move ok
nOkdir2		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRSE,x		get next board position
		bmi	nOkdir3		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nO_k	if blank, move ok
nOkdir3		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRSW,x		get next board position
		bmi	nOkdir4		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nO_k	if blank, move ok
nOkdir4		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRN,x		get next board position
		bmi	nOkdir5		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nO_k	if blank, move ok
nOkdir5		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRS,x		get next board position
		bmi	nOkdir6		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nO_k	if blank, move ok
nOkdir6		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRE,x		get next board position
		bmi	nOkdir7		if off the board, next direction
		lda	b,x		see what's on the board here
		beq	return_nO_k	if blank, move ok
nOkdir7		inc	dir		ready for next direction
		ldx	frm		index from position
		ldx	DIRW,x		get next board position
		bmi	nOcastle1	if off the board, next direction
		lda	b,x		see what's on the board here
		bne	nOcastle1	if not blank, try first castle move
return_nO_k	stx	Okp		a capture, save new king position
		txa			save TO position in A
		ldx	l		indicate cannot castle
		clr	sscf,x
		tax			get TO position and fall through
		bra	return_nO_x	return the move
nOcastle1	inc	dir		ready for other castle move
		ldx	l		get stack pointer
		lda	sscf,x		get castle flag
		beq	nOk_pos		can't castle, next board position
		ldx	frm		get from position
		cpx	#60		kings location?
		bne	nOk_pos		if not, next location
		lda	BPOS56		get the piece at location 57
		cmp	#O_rook		check if it is a rook
		bne	nOcastle2	if not, next direction
		lda	BPOS57		location 58
		bne	nOcastle2	not blank, next direction
		lda	BPOS58		location 59
		bne	nOcastle2    	not blank, next direction
		lda	BPOS59		location 60
		bne	nOcastle2	not blank, next direction
		ldx	#60		location 61
		jsr	O_attacked	if this location attacked
		beq	nOcastle2	then next direction
		ldx	#59		location 60
		jsr	O_attacked	if this location attacked
		beq	nOcastle2	then next direction
		lda	#-1		indicate queen side ie -ve
		sta	mf		through move flag
		ldx	#58		ok, after all that, castle is ok
		bra	return_nO_k	return the move
nOcastle2	inc	dir		to avoid an infinite loop
		lda	BPOS63		get the piece there at location 64
		cmp	#O_rook		check if it is a rook
		bne	nOk_pos		if not, next direction
		lda	BPOS62		location 63
		bne	nOk_pos		not blank, next direction
		lda	BPOS61		location 62
		bne	nOk_pos		not blank, next direction
		ldx	#61		location 62
		jsr	O_attacked	if this location attacked
		beq	nOk_pos		then end of king moves
		ldx	#60		location 61
		jsr	O_attacked	if this location attacked
		beq	nOk_pos		end of king moves
		lda	#1		king side move flag +ve
		sta	mf		indicate through move flag
		ldx	#62		ok, after all that, castle is ok
		bra	return_nO_k	return with move in a
nOk_pos		jmp	next_nO_pos	try the next board position
*
* return with move in X
*
return_nO_x	stx	pto		x has pto, so save it
		lda	temp		get the piece on the move
		sta	b,x		put it at its destination
		ldx	frm		get FROM position
		clr	b,x             clear the square there
		lda	mf		test for castle moves
		beq	nO_normal	no castle moves, check move is legal
		bmi	cO_queen_cas	mf is -ve a queen side castle
*
* king side castle, still need to move the rook
*
		lda	#O_rook		the rook
		sta	BPOS61		put rook in its place
		clr	BPOS63          clear original position of rook
		bra	nO_normal	continue, check move is legal
*
* queen side castle, still need to move the rook
*
cO_queen_cas	lda	#O_rook		the rook
		sta	BPOS59		put rook in its place
		clr	BPOS56          clear original position of rook
*
* check if valid move, ie not left king in check
*
nO_normal	ldx	Okp             get kings position
		jsr	O_attacked      see if computer is in check
		bne	nO_moveok	not in check, move ok
*
* king in check, not a valid move, restore board and try
* another move
*
n_restore	jsr	restore_nO	restore opponents non capture move
		jmp	nO_new_move	try another move
*
* a valid move so return it
*
nO_moveok       ldx	l		get search level (stack pointer)
		lda	mf		save move flag
		sta	ssmf,x
		lda     pto		save position to
		sta	spto,x		back on the search stack
		lda	dir		save dir
		sta	sdir,x
		lda	frm		save frm
		sta	sfrm,x
		clr	sstp,x		there is no captured piece!
		clc			indicate a valid capture
		rts

*****************************************************************************
*****************************************************************************
*                                                                           *
* Routines to restore the board after a move is made.                       *
*                                                                           *
*****************************************************************************
*****************************************************************************

*****************************************************************************
*                                                                           *
* restore computers move                                                    *
*                                                                           *
*****************************************************************************
res_C_move	lda	sfrm,x		get FROM position off stack
		sta	frm		save for restore
		lda	spto,x		get TO position off stack
		sta	pto		save for restore
		lda	sstp,x		get captured piece off stack
		sta	stp		save for restore
		lda	ssmf,x		get move flag off stack
		sta	mf		save for restore
		ldx	pto		get piece on move
		lda	b,x
		sta	temp		save for restore
		ldx	l		get stack pointer
		lda	scap,x		restore capture or non capture move?
		beq	restore_nC	non capture, fall through to capture

*****************************************************************************
*                                                                           *
* restore computers capture move                                            *
*                                                                           *
*****************************************************************************
restore_cC	lda	mf		get special moves flag
		beq	cC_restore_norm	nothing special, a normal restore
		bmi	cC_restore_prom	restore promotion move
		clr	stp		restore an en passan move
		lda	pto		get pawn position
		sub	#8
		tax			index it
		lda	#O_pawn		put the opponent pawn back
		sta	b,x
		bra	cC_restore_norm	continue with restore
cC_restore_prom	lda	stp		restore captured piece
		add	#8
		sta	stp
		lda	#C_pawn		cause pawn to be restored
		sta	temp		and fall through to rest of restore

cC_restore_norm	ldx	l		get level (stack pointer)
		lda	sskp,x		get saved king position
		sta	Ckp		restore kings position
		lda	sscf_2,x	get castle flag
		sta	sscf,x		restore castle flag
		ldx	frm		get FROM square
		lda	temp		get piece on the move
		sta	b,x		put it back on FROM square
		ldx	pto		get TO square
		lda	stp		get captured piece
		sta	b,x		put it back on TO square
		rts

*****************************************************************************
*                                                                           *
* restore computers non capture move		                            *
*                                                                           *
*****************************************************************************
restore_nC	lda	mf		get special moves flag
		beq	nC_restore_norm	nothing special, a normal restore
		bmi	nC_restore_qu	restore queen side castle move
		lda	#C_rook		restore king side castle move
		sta	BPOS7		put rook in original position
		clr	BPOS5		clear the rook from the board
		bra	nC_restore_norm	continue with rest of restore
nC_restore_qu	lda	#C_rook		restore queen side castle move
		sta	BPOS0		put rook in original position
		clr	BPOS3		clear the rook from the board

nC_restore_norm	ldx	l		get level (stack pointer)
		lda	sskp,x		get saved king position
		sta	Ckp		restore kings position
		lda	sscf_2,x	get castle flag
		sta	sscf,x		restore castle flag
		ldx	frm		get FROM square
		lda	temp		get piece on the move
		sta	b,x		put it back on FROM square
		ldx	pto		get TO square
		clr	b,x             clear destination square
		rts

*****************************************************************************
*                                                                           *
* restore opponents move                                                    *
*                                                                           *
*****************************************************************************
res_O_move	lda	sfrm,x		get FROM position off stack
		sta	frm		save for restore
		lda	spto,x		get TO position off stack
		sta	pto		save for restore
		lda	sstp,x		get captured piece off stack
		sta	stp		save for restore
		lda	ssmf,x		get move flag off stack
		sta	mf		save for restore
		ldx	pto		get piece on move
		lda	b,x
		sta	temp		save for restore
		ldx	l		get stack pointer
		lda	scap,x		restore capture or non capture move?
		beq	restore_nO	non capture, fall through to capture


*****************************************************************************
*                                                                           *
* restore opponents capture move                                            *
*                                                                           *
*****************************************************************************
restore_cO	lda	mf		get special moves flag
		beq	cO_restore_norm	nothing special, a normal restore
		bmi	cO_restore_prom	restore promotion move
		clr	stp		restore an en passan move
		lda	pto		get pawn position
		add	#8
		tax			index it
		lda	#C_pawn		put the opponent pawn back
		sta	b,x
		bra	cO_restore_norm	continue with restore
cO_restore_prom	lda	stp		restore captured piece
		sub	#8
		sta	stp
		lda	#O_pawn		cause pawn to be restored
		sta	temp		and fall through to rest of restore

cO_restore_norm	ldx	l		get level (stack pointer)
		lda	sskp,x		get saved king position
		sta	Okp		restore kings position
		lda	sscf_2,x	get castle flag
		sta	sscf,x		restore castle flag
		ldx	frm		get FROM square
		lda	temp		get piece on the move
		sta	b,x		put it back on FROM square
		ldx	pto		get TC square
		lda	stp		get captured piece
		sta	b,x		put it back on TO square
		rts

*****************************************************************************
*                                                                           *
* restore opponents non capture move		                            *
*                                                                           *
*****************************************************************************
restore_nO	lda	mf		get special moves flag
		beq	nO_restore_norm	nothing special, a normal restore
		bmi	nO_restore_qu	restore queen side castle move
		lda	#O_rook		restore king side castle move
		sta	BPOS63		put rook in original position
		clr	BPOS61
		bra	nO_restore_norm	continue with rest of restore
nO_restore_qu	lda	#O_rook		restore queen side castle move
		sta	BPOS56		put rook in original position
		clr	BPOS59          clear the rook from the board

nO_restore_norm	ldx	l		get level (stack pointer)
		lda	sskp,x		get saved king position
		sta	Okp		restore kings position
		lda	sscf_2,x	get castle flag
		sta	sscf,x		restore castle flag
		ldx	frm		get FROM square
		lda	temp		get piece on the move
		sta	b,x		put it back on FROM square
		ldx	pto		get TO square
		clr	b,x		clear destination square
		rts			and return

*****************************************************************************
*                                                                           *
* IO Control Routine                                                        *
*                                                                           *
*****************************************************************************
*
* This routine displays information on the LED's while waiting for a key
* press. Program status flags are set depending on which key is pressed.
*
* If after about 8.5 minutes (7.8125ms*65536) a key has not been pressed,
* stop mode is entered until the OK key is pressed (which causes an IRQ).
*
* If ALL three keys are pressed simultaneously, this causes the game to
* restart from scratch.
*
* NOTE: Pressing OK will set the IRQ D flip flop and thus cause an IRQ
* immediately the interrupt bit is cleared. See comp_move for how this is
* handled.
*
* Inputs:
*	MEMORY	BIT	DESCRIPTION
*	group1	0:7	LEDs 1 to 8, lsb is LED 1 (A-H)
*	group2	0:7	LEDs 9 to 16, lsb is LED 9 (1-8)
*	group3	0:1	LEDs 17 to 18, lsb is LED 17 (FROM, TO)
*	group4	2:3	LEDs 19 to 20, lsb is LED 19 (DRAW, MATE)
*       flash_grp       Indicate the group to flash (0-3), or -1 if none
*
* Outputs:
*	KEY	FLAG
*	pub1	C set
*	pub2    C clear
*	pub3	Z set

group1		equ	temp4		what's to be display on LED group 1
group2		equ	temp5		what's to be display on LED group 2
group3		equ	temp6		what's to be display on LED group 3
group4		equ	temp7		what's to be display on LED group 4
flash_cnt	equ	temp8		flash rate counter
group_sel	fcb     %00000111	select group 1
		fcb     %00001011	select group 2
		fcb     %00001110	select group 3
		fcb     %00001101	select group 4
*
* initialise key debounce flags
*
user_io		lda	ACRHI		setup 7.8ms delay for multiplexing
		ldx	ACRLO
		add	#1024/256	set OCR to current timer value
		sta	OCRHI
		lda	TSR		if OCR set, next instruction clears it
		stx	OCRLO		init output compare
		lda	#-1		set debounce flags to -1
		sta	temp2		key release debounce flag
		sta	temp3		key pressed debounce flag
		sta	flash_cnt	clear flash rate counter
*
* initialise auto shut down timers
*
		clr	gametime_hi	used here for auto shut down
		clr	gametime_lo
*
* display the next group
*
mul_loop	clr	src_port	ensure no display while switching
		inc	flash_cnt	count next loop delay
		lda	in_port		get key value
		and	#$7		mask all but keys
		bne	mul_tiplex      not all keys are pressed, so continue
		jmp	reset		all keys are pressed, start new game
mul_tiplex	lda	multiplex	next group number to display
		inca
		and	#3		range is 0-3
		sta	multiplex	save it
		cmp     flash_grp	should this group flash?
		bne	mul_show_grp	no, display this group
		brset	fl_rate,flash_cnt,mul_delay only display at fl_rate
mul_show_grp	tax			index it
		lda	group1,x	get the group data
		ldx	group_sel,x	get multiplexing group selector
		stx	snk_port	select the group
		sta	src_port	display group information
*
* delay for 7.8ms
* which is 4096 timer ticks
*
mul_delay	brclr	6,TSR,*		wait for RTIF
		lda	OCRLO		setup OCR for next time
		tax			remember low byte
		lda	OCRHI		get the high byte
		add	#1024/256	adjust for next compare
		sta	OCRHI		update high byte
		stx	OCRLO		update low byte and clear OCR flag
*
* process key release and debounce
*
		lda	temp2		test key release flag
		bpl	mul_krd		not -ve, doing key press or debounce
		lda	in_port		get key value
		ora	#$f8		mask off all but keys
		coma			keys are active low
		bne	mul_loop	a key is still pressed so loop
		lda	#debounce	key has been released, need to
		sta	temp2		debounce it
		bra	mul_loop	debounce for 50ms
*
* debounce key release
*
mul_krd		beq	mul_kpress	if 0, process key press
		dec	temp2		continue with debounce until 0
		bra	mul_loop
*
* process key press and debounce
*
mul_kpress	lda	temp3		debounce key press?
		bpl	mul_kpd		yes
		lda	in_port		get key value
		ora	#$f8		mask off all but keys
		coma			keys are active low
		bne	kp_debounce	if key pressed then debounce
		inc	gametime_lo	if no key press for 65536 loops
		bne	mul_loop
		inc	gametime_hi
		bne	mul_loop
		clr	src_port	LEDs off while sleeping
		clr	snk_port	bring all output ports low
		com	sleeping	indicate to IRQ routine proc. state
		stop			go to sleep, note I flag is cleared
		sei			no more interrupts
		clr	sleeping	reset sleeping flag
		jmp	user_io		after OK pressed, wait for release
kp_debounce	lda	#debounce	key has been pressed, need to
		sta	temp3		debounce it
		bra	mul_loop	debounce for 50ms
*
* debounce key press
*
mul_kpd		dec	temp3		delay for key press debounce
		bne     mul_loop
*
* get and decode the key press and return
*
		lda	#127		ensure Z is clear
		brclr	0,in_port,mul_pb1 push button 1 is pressed, SHIFT
		brclr	1,in_port,mul_pb2 push button 2 is pressed, NEXT
		brclr	2,in_port,mul_pb3 push button 3 is pressed, OK
		jmp	mul_loop	this should never happen!
mul_pb1		sec			set C flag, SHIFT
mul_pb2		rts			if NEXT/FROM/TO, return C clear
mul_pb3		clra			set Z flag, OK
		rts

*****************************************************************************
*                                                                           *
* Routine to initialise groups for display of opponents move.               *
*                                                                           *
*****************************************************************************
*
* x points to FROM or TO position in page zero
*
opp_init_disp	lda	#$1		init group 1
		sta	group1		that is, the left most LED
		lda	#$80		init group 2 to $80
		sta	group2		that is the right most LED
		lda	0,x		get TO/FROM
		and	#7		get x coordinate
		tst	O_white		opponent white?
		bne	opp_init_lpx	yes, don't subtract from 7
		nega			subtract from 7
		add	#7
opp_init_lpx	tsta
		beq	opp_init_y	if zero, exit now
		lsl	group1		light next LED
		deca			keep rotating until
		bne	opp_init_lpx	x coord reached zero
opp_init_y	lda	0,x		get TO/FROM
		lsra			divide by 8 to get y coordinate
		lsra
		lsra
		beq	opp_init_exit	if zero, exit now
opp_init_lpy	lsr	group2		light next LED
		deca			keep rotating until
		bne	opp_init_lpy	y coord reached zero
opp_init_exit	rts

*****************************************************************************
*                                                                           *
* process IRQ interrupt                                                     *
*                                                                           *
*****************************************************************************
*
* IRQ occurs when interrupts are enabled (during search) and the OK button
* was pressed
*
* if not searched 2 levels yet, exit immediately
* otherwise, disable timer interrupts
* let the progress LED stay on for its usual time, then turn it off
* reset the stack
* make computers move
*
irq_int		lda	sleeping	has processor been stopped
		bne	irq_return	yes, return to io routine
		lda	to_level	search level<=2?
		cmp	#lb+2
		bhi	terminate	no, terminate the search
irq_return	rti			yes, continue searching
*
* Main search has been interrupted by an impatient opponent!
* So reset the stack, turn the LED off and get the best move from the
* last full iteration
*
terminate	rsp			init stack for making computers move
		jmp	up_board	update the board

*****************************************************************************
*                                                                           *
* process TIMER interrupt                                                   *
*                                                                           *
*****************************************************************************
*
* we branch here every 62.5ms during searching for
* best move for computer
* we want to flash the MATE LED for 62.5ms every couple of seconds
*
* TIMER interupt occurs every 62.5 ms
* after two seconds update the amount of time used so far for the search
* which depends on the level of play entered by the opponent
* if time is up, make the move from the last full iteration
*
*
timer_int	lda	TSR		read TSR with OCF set
		jsr	settimer	clear OCF and reset timer
		dec	fled		FLED counts 125ms intervals
		bne	led_off		not zero, turn LED off
		lda	#flash		it's zero, reset interval counter
		sta	fled
		bset	3,src_port	turn LED on
		lda	half_moves	if in opening search
		cmp	#OP_MOVES	then continue the search until
		blo     cont_search	completed
*
* a second has elapsed, so update the number of seconds the search has
* taken so far and, depending on the level of game, continue the search
* or get the best move from the last full iteration and terminate the
* search
*
		dec	gametime_lo	update game time low byte
		bne	cont_search	if not zero, continue the search
		lda	gametime_hi	if high byte zero, terminate search
		beq	terminate	if reached zero, terminate search
		dec	gametime_hi	game time high byte
cont_search	rti			back to search
led_off		clr	src_port	LED off
		rti

*****************************************************************************
*                                                                           *
* display computers move                                                    *
*                                                                           *
*****************************************************************************
*
* if computer is white, then init group 1 to 80, group2 to 80
* if computer is black, then init group 1 to 1, group2 to 80
*
com_show        sta	temp		FROM/TO position
		stx	group3		FROM/TO LED selection
		lda	#$80		init group 2
		sta	group2
		ldx	O_white		is opponent playing white?
		beq	com_is_white	no, computer is white
		lda	#$1		computer is playing black
com_is_white	sta	group1		init group 1
		lda	temp		get FROM/TO position
		and	#7		get x coordinate
		beq	com_ogr1	if zero, exit now
com_conv1a	ldx	O_white		is opponent playing white?
		beq	com_led_white	no, computer is white, do lsr
		lsl	group1		light next LED
		bra	com_nxt_led
com_led_white	lsr	group1		light next LED
com_nxt_led	deca			keep rotating until
		bne	com_conv1a	x coord reached zero
com_ogr1	lda	temp		get FROM/TO again
		lsra			divide by 8 to get y coordinate
		lsra
		lsra
		beq	com_show1	if zero, exit now
com_conv1b	lsr	group2		light next LED
		deca			keep rotating until
		bne	com_conv1b	y coord reached zero
*
* set FROM/TO LED and display FROM/TO position
*
com_show1	jsr     user_io		display groups, wait for key press
		rts			exit on any key pressed

*****************************************************************************
*                                                                           *
* Initialise to get opponents move                                          *
*                                                                           *
*****************************************************************************
init_opp_move	ldx     #lb+1		init stack to odd number
		stx	l
		lda	pcf0		get computers last move
		sta	sfrm_1,x	for opponent to check for ep
		lda	pct0		posibility
		sta	spto_1,x
		lda	sscf0_1         init castle flag
		sta	sscf,x
		lda	Okp		save kings position
		sta	sskp,x
		lda	#$ff		enable capture moves
		sta	scap,x		set capture flag
		lda	#63		set FROM position to start address
		sta	sfrm,x
		clr	sdir,x		clear direction
		rts

*****************************************************************************
*                                                                           *
* set OCR timer for interrupt every 62.5ms                                  *
*                                                                           *
*****************************************************************************
settimer	lda	ACRHI		get current timer value
		add	#$80            adjust for 62.5ms delay
		sta	OCRHI
		lda	ACRLO		OCF will be cleared
		sta	OCRLO
		rts

*****************************************************************************
*                                                                           *
* Delay for 0.125 of a second                                               *
*                                                                           *
*****************************************************************************
sleep0125	clr	src_port	blank display while asleep
		bsr	settimer
		brclr	6,TSR,*		wait for output compare flag
		bsr	settimer
		brclr	6,TSR,*		wait for output compare flag
*		ldx	OCRLO		clear OCR flag
*		stx	OCRLO		required for simulator to clear OCF
		rts

*****************************************************************************
*                                                                           *
* Setup and main control                                                    *
*                                                                           *
*****************************************************************************
* Uses
*	MEMORY	BIT	DESCRIPTION
*	inttype	0	progress LED, timer set for 1 second interrupt
*	inttype	1	progress LED, timer set for 100ms interrupt
*	inttype	2	multiplex, timer set for 10ms delay
*
* micro controller jumps here after reset
* first thing to do is set up IO ports
* PORT B is an input
* RESET
*
reset           sei			restart(0), no interrupts
		rsp			stack could be in use so reset
		lda	#$80		option register, set for RAM 0 init
		sta	OPTR		regions and IRQ edge only interrupt
		jsr	set_up_all	set up board and io ports
		ldx	#$c0		enable BOTH RAM banks
		stx	OPTR
		lda	#$ff		castle flags set true
		sta	sscf0_2
		sta	sscf0_1
		sta	sfrm0_1		init first opp_move so ep
		sta	spto0_1		check is negative
		sta	pcf0		init first computer move
		sta	pct0		so ep check is negative

*****************************************************************************
*                                                                           *
* Input the level and who is to play white.                                 *
*                                                                           *
*****************************************************************************
gamelevel	equ	temp
*
* get game level
* until OK is pressed
*

**** FOR SIMULATION *****
*		CLR	gamelevel
*		BRA	SET_LEV
**** FOR SIMULATION *****

		clr	gamelevel	init level to 0
		clr	group1		group 1 to display nothing
		clr	group3		group 3 to display nothing
		lda	#1		group 2 left LED on ie level 1
		sta	group2
		sta	flash_grp	group 2 to flash
		lda	#4		group 4 LEVEL LED on
		sta	group4
		bra	lev_disp	display the groups
lev_loop	lsl	group2		light the next LED
		bcc	lev_nlev	if not complete circle, next level
		rol	group2		rotate C into lsb
lev_nlev	inc	gamelevel	next level
lev_disp	jsr	user_io		LEDs on and wait for key press
		bcs	lev_loop	SHIFT pressed, go to next level
		bne	lev_disp	until OK pressed
SET_LEV		lda	#11		init max game time to 11*2^gamelevel
		sta	totaltime_lo
		clr	totaltime_hi
		lda	gamelevel	adjust level to be 0-7
		and	#7		this is number of times to multiply

**** FOR SIMULATION *****
*		BRA	SIM_SKIP_1
**** FOR SIMULATION *****

		beq	ask_white	if zero, exit now
lev_times_2	lsl	totaltime_lo	max game time by 2
		rol	totaltime_hi
		deca
		bne	lev_times_2
*
* do a courtesy delay for 0.125 second with all LEDs off
*
ask_white	bsr	sleep0125	sleep for a while
*
* ask who is to play white?
* until OK is pressed
*

**** FOR SIMULATION *****
*SIM_SKIP_1	NOP
**** FOR SIMULATION *****

		clr	group4		turn LEVEL LED off
		lda	#2		group 3 to flash
		sta	flash_grp
		clr	O_white		initially assume opponent is black

**** FOR SIMULATION *****
*		clr	O_white		initially assume opponent is black
*		COM	O_white
*		BRA	PLAY_IT
**** FOR SIMULATION *****

**** FOR SIMULATION *****
who_loop	lda	#1		init to 1 (ie WHITE)
		com	O_white		switch to other colour
		bne	who_store	if not zero opponent wants WHITE
		lda	#2		otherwise, opponent wants BLACK
who_store	sta	group3
who_disp	jsr	user_io		display groups, wait for key press
		bcs	who_loop	pressed SHIFT
		bne	who_disp	until pressed OK
*
* OK, play the game
*
		lda	O_white		true if not zero
		beq	comp_move	computer is white, do computers move
		jmp	opp_move_ok	opponent is white, go get opp move

*****************************************************************************
*                                                                           *
* Find the best move for the computer.                                      *
*                                                                           *
*****************************************************************************
*
* do courtesy delay for 0.125 second with
* all LEDs off
comp_move	clr	src_port	display off
		brclr	2,in_port,*	wait for OK release to avoid IRQ
		jsr	sleep0125	debounce and courtesy delay

**** FOR SIMULATION *****
*PLAY_IT	NOP
**** FOR SIMULATION *****

**** FOR SIMULATION *****
*PLAY_IT        lda	#$3e
*		sta	Okp
*		lda	#$b
*		sta	Ckp
*		clra
*		sta	sscf0_1
*		JMP	opp_move_ok
**** FOR SIMULATION *****

		clr	TCR		no interrupts from timer
		clr	to_level	ensure IRQ routine just returns
		cli			pressing OK has set IRQ flip flop
		sei			processing IRQ will clear it
*
* init to make computers move
* set maximum search time
*
		lda	totaltime_hi	high byte of maximum search time
		sta	gametime_hi	init high byte search time
		lda	totaltime_lo	low byte of maximum search time
		sta	gametime_lo	init low byte search time
		jsr	settimer	set timer for 62.3 ms delay
		lda	#$40		enable OCR interrupt
		sta	TCR
		lda	#flash		8*125ms = 1 second
		sta	fled		save as interval counter
		clr	src_port	flash MATE LED every second
		lda	#$fd		but start with it off, 1101
		sta	snk_port	for 125ms
*
* determine the material balance
*
		lda     #O_king		init to -ve value of a king
		sta	Cmat
		sta	Omat
		ldx	#63
up_mat_loop	lda	b,x		get a piece
		bmi	Opiece		if -ve it's opponents
		add	Cmat		it's +ve, update computers material
		sta	Cmat
		bra	mat_loop
Opiece          nega			update opponents material
		add	Omat
		sta	Omat
mat_loop	decx			next board position
		bpl	up_mat_loop
		lda	Cmat		difference is material balance
		sub	Omat
		sta	bal
*
* initialise iteratively deepening search
* forced exits are not permitted during opening moves and first 2 ply
* of the main search
*
		lda	#lb+1		init TO_LEVEL to 1
		sta	to_level
		lda	half_moves	are we still doing opening moves?
		cmp	#OP_MOVES	there are at most OP_MOVES half moves
		bhs	main_search	no, do middle/end game search
		cli			enable interrupt from timer, IRQ
		jsr	alphabeta       get computers move
		sei			disable interrupts while updating
		jmp	up_board	and update the board
main_search	cli			enable interrupt from timer, IRQ
		jsr	alphabeta       get computers move
		sei			disable interrupts while saving move
		lda	mflag		check the move flag
		beq	save_mv		save the move if ok
		cmp	#know_move	an exact score?
		beq	save_mv		yes, save move
		jmp     comp_led	no, must be end of game
*
* Save the best move found after each iteration so that the search
* can be interrupted at any time but there is always the best move from
* at least the last full iteration. Of course interrupts must be disabled
* while saving the position after the last iteration.
*
save_mv		lda	to_level	check level just searched
		cmp	#lb+2		is at least 2
		bls	save_last	no, remember this as previous move
		lda	i_frm		move same as before?
		cmp	l_frm
		bne	save_last	no, but remember it
		lda	i_pto
		cmp	l_pto
		bne	save_last	FROM is same but TO is not...
		lda	m0		score greater or same as before?
		cmp	l_score
		bpl	up_board	yes, exit search now
save_last       lda	m0		score from last iteration
		sta	l_score
		lda	i_frm		FROM position
		sta	pcf0
		sta	l_frm
		lda	i_pto		TO position
		sta	pct0
		sta	l_pto
		lda	i_cf		castle flag
		sta	bcf
		lda	i_kp		kings position
		sta     bkp
		ldx	#63		save the board
save_board	lda	b,x
		sta	sb,x
		decx
		bpl	save_board
		ldx	i_frm		update the saved board with the
		lda	sb,x		best move after this iteration
		clr	sb,x
		tst	i_cap		a capture move?
		bne	save_cap	yes, update board with capture move
		tst	i_mf		no, check for castling move
		beq	save_cont	an ordinary move
		bmi	save_queen	mf is -ve a queen side castle
		ldx	#C_rook		the rook
		stx	SBPOS5		put rook in its place
		clr	SBPOS7		clear original position of rook
		bra	save_cont	continue
save_queen	ldx	#C_rook		the rook
		stx	SBPOS3		put rook in its place
		clr	SBPOS0		clear original position of rook
		bra	save_cont	continue
save_cap	tst	i_mf		get the move flag
		beq	save_cont	nothing special, normal capture
		bpl	save_epas	ep pawn capture
		lda	#C_queen	pawn promotion
		bra	save_cont
save_epas	lda	i_pto		en passan
		sub	#8		remove captured pawn
		tax
		clr	sb,x
		lda	#C_pawn
save_cont	ldx	i_pto
		sta	sb,x
		lda	mflag		if an exact score
		bne	up_board	exit now
		inc	to_level	next level
		lda	to_level	max level reached?
		cmp	#SEARCH_STACK+lb
		bhs	up_board	yes, update the board
		jmp	main_search	else, continue searching
*
* update the board with the best move found
*
up_board	lda	bcf		update castle flag
		sta	sscf0_2
		lda	bkp		update kings position
		sta	Ckp
		ldx	#63		update the board
update_board	lda	sb,x
		sta	b,x
		decx
		bpl	update_board

		inc	half_moves	one half move is made
comp_led        clr	TCR		no interrupts from timer
		lda	fled		LED on?
		cmp	#flash
		bne	comp_result	no, leave it off
		brclr	6,TSR,*		wait for OCF to set
		ldx	OCRLO		clear OCF
		stx	OCRLO		required for simulator to clear OCF
		clr	src_port	ensure LED is off
comp_result	ldx	#8		assume computer is check and mate
		brset	0,mflag,comp_mate =1, mate
		brset	1,mflag,comp_draw =2, draw
		bra	opp_move	not end of game, get opponents move
comp_draw	ldx	#4		draw
comp_mate	stx	group4		indicate result on LED group 4
		clr	group1		other LEDs are off
		clr	group2
		clr	group3
		lda	#3		flash result on LED group 4
		sta	flash_grp
comp_show_last	jsr     user_io		display groups, wait for key press
		bne	comp_show_last	if not pressed OK, continue display
		jmp	reset		pressed OK, restart
*
* check if opponent has any moves to make
*
opp_move	clr	group4		init not draw and not mate
		lda	s_Okp		get saved king position
		sta	Okp		restore Okp after interrupt
		jsr	init_opp_move	init to get opponent move
*
* check if there are any moves for opponent to make
*
		jsr	next_O_move	try a move
		bcc	com_res		there are moves, display computers
*
* there were no legal moves at all, so if opponents king
* is attacked then opponent is in check mate, otherwise
* we indicate a draw
*
		lda	#3		flash group 4, ie DRAW or MATE
		sta	flash_grp
		lda	#4		init to display draw
		sta	group4
		ldx	Okp		see if opponent attacked
		jsr	O_attacked
		bne	com_disp	not attacked, display draw
		lda	#8		otherwise, check mate!
		sta	group4		save the display result
		bra	com_disp	display result and computers move
*
* display computers move until OK pressed
* convert FROM to display format
*
com_res         ldx	l		restore the stack
		jsr	res_O_move	restore the board

* FOR SIMULATION
*		BRA	SKIP_COM_DISP
* FOR SIMULATION

		lda	#$ff		disable LED flash function
		sta	flash_grp
com_disp	lda	pcf0		set up to display FROM
		ldx	#1		set up to light FROM LED
		jsr	com_show	display FROM position
		lda	pct0		set up to display TO
		ldx	#2		set up to light TO LED
		jsr	com_show	display TO position
		bne	com_disp	pressed NEXT, display FROM again
		lda	group4		if group 4 not zero, end game
		beq	opp_move_ok	it's zero, continue
		jmp	reset		restart game
*
* get opponents move
* until OK is pressed
* do courtesy delay for 0.25 second with
* all LEDs off
*

* FOR SIMULATION
*opp_move_ok	NOP
* FOR SIMULATION

opp_move_ok	jsr	sleep0125	sleep for a little while
*
* init for opponents move
* use Cmat and Omat, since not used outside search
*

* FOR SIMULATION
*SKIP_COM_DISP	NOP
* FOR SIMULATION

opp_from	equ	Cmat		opponents position from
opp_to		equ	Omat		opponents position to
		lda	#1		if opp is white, init to 1, black -1
		sta	temp
		lda	#$38		when opponent is white this is A1
		ldx	O_white		is opponent white?
		bne	opp_init_ft	yes, init FROM and TO positions
		lda	#$FF		opponent is black, set to -1
		sta	temp
		lda	#$3F		when opponent is black this is A1
opp_init_ft	sta	opp_from	init input position FROM and TO
		sta	opp_to

* FOR SIMULATION
*		JMP	opp_try
* FOR SIMULATION

*
* setup to display and get opp_from
* if opponent is black x coordinate is reversed by subtracting it from 7
*
opp_loop	ldx	#opp_from	point to opp_from
		jsr	opp_init_disp	init groups for display
*
* set FROM LED and display and get FROM X coordinate
*
		clr	flash_grp	flash group 1 (A-H) LEDs
		lda	#1		light FROM LED in group 3
		sta	group3
		bra	opp_f_show1	display and get response
opp_f_loop1     lsl	group1		light the next LED (A-H)
		bcc	opp_f_ng1	if not complete circle, next
		rol	group1		rotate C into lsb lighting LED A
		lda	opp_from	reset X to A1 X position
		ora	#$7
		tst	O_white		depending on opponent colour
		beq	opp_f_init	if black set to ?????111
		and	#$f8		if white set to ?????000
opp_f_init	sta	opp_from
		bra     opp_f_show1	and display it
opp_f_ng1	lda	opp_from	add/subtract 1 from X coordinate
		add	temp		depending on whether opponent is
		sta	opp_from	white/black
opp_f_show1	jsr     user_io		display groups while waiting for user
		bcs	opp_f_loop1	pressed SHIFT, move to next X
		bne	opp_f_next	pressed NEXT, move to Y coordinate
		jmp	opp_try		pressed OK, try the opponents move
opp_f_next      lda	#1		flash group 2 (1-8) LEDs
		sta	flash_grp
		bra	opp_f_show2	pressed NEXT
*
* display and get FROM Y coordinate
*
opp_f_loop2     lsl	group2		light the next LED (1-8)
		bcc	opp_f_ng2	if not complete circle, next
		rol	group2		rotate C into lsb lighting LED A
		lda	opp_from	set Y coordinate to 7
		ora	#$38
		sta	opp_from
		bra     opp_f_show2	and display it
opp_f_ng2	lda	opp_from	subtract 1 from Y coordinate
		sub	#8
		sta	opp_from
opp_f_show2	jsr     user_io		display groups while waiting for user
		bcs	opp_f_loop2	pressed shift, move to next X
		beq	opp_try		pressed OK, try the opponents move
*		jsr	sleep0125	short courtesy delay
*
* setup to display and get opp_to TO position
*
		ldx	#opp_to		point to opp_to
		jsr	opp_init_disp	init groups for display
*
* set TO LED and display and get TO X coordinate
*
		clr	flash_grp	flash group 1 (A-H) LEDs
		lda	#2		light TO LED in group 3
		sta	group3
		bra	opp_t_show1	display and get response
opp_t_loop1     lsl	group1		light the next LED (A-H)
		bcc	opp_t_ng1	if not complete circle, next
		rol	group1		rotate C into lsb lighting LED A
		lda	opp_to		reset X to A1 X position
		ora	#$7
		tst	O_white		depending on opponent colour
		beq	opp_t_init	if black set to ?????111
		and	#$f8		if white set to ?????000
opp_t_init	sta	opp_to
		bra     opp_t_show1	and display it
opp_t_ng1	lda	opp_to		add/subtract 1 from X coordinate
		add	temp		depending on whether oppenent is
		sta	opp_to		white/black
opp_t_show1	jsr     user_io		display groups while waiting for user
		bcs	opp_t_loop1	pressed SHIFT, move to next X
		beq     opp_try		pressed OK, try the opponents move
		lda	#1		flash group 2 (1-8) LEDs
		sta	flash_grp
		bra	opp_t_show2	display and get Y coordinate
*
* display and get TO Y coordinate
*
opp_t_loop2	lsl	group2		light the next LED (1-8)
		bcc	opp_t_ng2	if not complete circle, next
		rol	group2		rotate C into lsb lighting LED A
		lda	opp_to		set Y coordinate to 7
		ora	#$38
		sta	opp_to
		bra     opp_t_show2	and display it
opp_t_ng2	lda	opp_to		subtract 1 from Y coordinate
		sub	#8
		sta	opp_to
opp_t_show2	jsr     user_io		display groups while waiting for user
		bcs	opp_t_loop2	pressed shift, move to next X
		beq	opp_try		if pressed OK, try the move
		jmp	opp_loop	continue inputting opponent move
*
* check if this is a valid move in this position
*
opp_try		jsr	init_opp_move	initialise for opponents move
opp_try_next	jsr	next_O_move	get the next move
		bcc	opp_test_move	not the last move, test it's legal
		jmp	opp_move_ok	no more moves, get opponent move
opp_test_move	lda	opp_from	test FROM
		cmp	frm
		bne	opp_res		not the same, restore move, try again
		lda	opp_to
		cmp	pto
		bne	opp_res		not the same, try again
		ldx	l		point to stack
		lda	frm		save frm at frm-1 for ep check
		sta	sfrm0_1
		lda	pto		same with pto
		sta	spto0_1
		lda	sscf,x		get castle flag
		sta	sscf0_1		update top castle
		lda	Okp		save kings position
		sta	s_Okp
		inc	half_moves	a half move made
		jmp	comp_move	don't restore board, get comp's reply
opp_res		ldx	l		restore the stack
		jsr	res_O_move	restore the board
		bra	opp_try_next	get another move

*****************************************************************************
*****************************************************************************
*                                                                           *
* The Alpha Beta Search Routines                                            *
*                                                                           *
*****************************************************************************
*****************************************************************************

*****************************************************************************
*                                                                           *
* beta part of ab search to minimise score after opponents move             *
* first check if there are any moves to make...                             *
*                                                                           *
*****************************************************************************
O_start		inc	l		increment the stack pointer
		ldx	l		get the stack pointer
		lda	#maxscore	initialise m to maximium score
		sta	m,x		save a parameter to init m
		lda	a_2,x		this is previous alpha/beta
		sta	a,x		save it for next level
		lda	sscf_2,x	get castle flag from last move
		sta	sscf,x		save it for this move
		lda	#$ff		enable capture moves
		sta	scap,x		set capture flag
		lda	#63		set FROM position to start address
		sta	sfrm,x
		clr	sdir,x		clear direction
		lda	Okp		save kings position
		sta	sskp,x
*
* check if there are any moves to make
*
		jsr	next_O_move	try a move
		bcc	O_move_ok	move is ok, continue search
*
* there were no legal moves at all, so if opponents king
* is attacked then opponent is in check mate, otherwise
* we indicate a draw
*
		clr	v		in case it's a draw, set score to 0
		ldx	Okp		see if oppenent attacked
		jsr	O_attacked
		bne	O_exit		not attacked, exit with draw score
		lda	#maxscore	otherwise, check mate!
		sta	v		save the score
		bra	O_exit		and return with it

O_move_ok	lda	bal		get material balance
		ldx	l		point to stack
		sub	sstp,x		update it with a possible capture
		sta	bal		save it back
		cpx	to_level	deep enough?
		blo	C_start		not yet, go lower
		ldx	spto,x		check if the position TO square
		jsr	O_attacked	is attacked?
		beq	O_adjust	yes, adjust score
		lda	bal		no, get current material balance
		sta	v		use this as the final score
		bra	O_res_bal	and restore bal
O_adjust	lda	bal		adjust bal for attacking piece
		sub	temp		this is the attacking piece
		sta	v		save this as the final score
O_continue	lda	bal		get material balance
O_res_bal	ldx	l		point to stack
		add	sstp,x		restore balance
		sta	bal		save it back

		lda	v		get the score
		cmp	m,x		check it against current minimum
		bpl	O_try_next	not smaller, so try next move
		lda	a_1,x		check it against last beta
		cmp	v
		bmi	O_new_min	if v is greater, still a new minimum
		jsr	res_O_move	yes, restore the board
		bra	O_exit		and exit

O_new_min	lda	v		get v back
		sta	m,x		new minimum
		sta	a,x		new alpha

O_try_next      jsr	res_O_move	restore the board
		jsr	next_O_move	get next move
		bcc	O_move_ok	move is legal
*
* no more moves, return the best score
*
		ldx	l		restore the stack pointer
		lda	m,x		get current best score
		sta	v		save it
O_exit		dec	l		decrement stack pointer, level
		ldx	l		if reached lb (a level of 0) exit
		cpx	#lb
		bhi	C_continue	otherwise continue
		rts

*****************************************************************************
*                                                                           *
* alpha part of ab search to maximise score after computers move            *
* first check if there are any moves to make...                             *
*                                                                           *
*****************************************************************************
C_start		inc	l		increment the stack pointer
		ldx	l		get the stack pointer
		lda	#minscore	initialise m to maximium score
		sta	m,x		save a parameter to init m
		lda	a_2,x		this is previous alpha/beta
		sta	a,x		save it for next level
		lda	sscf_2,x	get castle flag from last move
		sta	sscf,x		save it for this move
		lda	#$ff		enable capture moves
		sta	scap,x		set capture flag
		lda	#63		set FROM position to start address
		sta	sfrm,x
		clr	sdir,x		clear direction
		lda	Ckp		save kings position
		sta	sskp,x
*
* check if there are any moves to make
*
		jsr	next_C_move	try a capture move
		bcc	C_move_ok	move is ok, continue search
*
* there were no legal moves at all, so if opponents king
* is attacked then opponent is in check mate, otherwise
* we indicate a draw
*
		dec	l		ready for exit (to O_continue)
		clr	v		in case it's a draw, set score to 0
		ldx	Ckp		see if opponent attacked
		jsr	C_attacked
		bne	O_continue	not attacked, exit with draw score
		lda	#minscore	otherwise, check mate!
		sta	v		save the score
		bra	O_continue	and return with it

C_move_ok	lda	bal		get material balance
		ldx	l		point to stack
		sub	sstp,x		update it with a possible capture
		sta	bal		save it back
		cpx	to_level	deep enough?
		bhs	C_nolow		yes, continue
		jmp	O_start		not yet, go lower
C_nolow		ldx	spto,x		check if the position TO square
		jsr	C_attacked	is attacked?
		beq	C_adjust	yes, adjust score
		lda	bal		no, get current material balance
		sta	v		use this as the final score
		bra	C_res_bal	and restore bal
C_adjust	lda	bal		adjust bal for attacking piece
		sub	temp		this is the attacking piece
		sta	v		save this as the final score
C_continue	lda	bal		get material balance
C_res_bal	ldx	l		point to stack
		add	sstp,x		restore balance
		sta	bal		save it back

		lda	m,x		get the current maximum
		cmp	v		check it against the score
		bpl	C_try_next	not smaller, so try next move
		lda	v		get score
		cmp	a_1,x		check it against last alpha
		bmi	C_new_min	if v is greater, still a new minimum
		jsr	res_C_move	yes, restore the capture move
		bra	C_exit		and exit

C_new_min	lda	v		get v back
		sta	m,x		new minimum
		sta	a,x		new alpha

C_try_next	jsr	res_C_move	restore the capture move
		jsr	next_C_move	get the next move
		bcc	C_move_ok	there are more moves
*
* no more moves, return the best score
*
		ldx	l		restore stack pointer
		lda	m,x		get current best score
		sta	v		save it
C_exit		dec	l		decrement stack pointer, level
		jmp	O_continue	and return

*****************************************************************************
*****************************************************************************
*                                                                           *
* Find the best move for the computer.                                      *
*                                                                           *
*****************************************************************************
*****************************************************************************
*	l	is stack pointer
*	mflag 	on exit inidcates check mate, draw or a normal move
*	a	stack of alpha beta values
*	m	stack of best values found so far
*
alphabeta	ldx	#lb		initialise the stack pointer
		stx	l
		lda	#maxscore	initialise a[-1] to max score
		sta	a_1,x
		lda	#minscore	init value for a[0]
		sta	m,x		initialise m[0] to min score
		sta	a,x		initialise alpha beta score
		sta	mb		initialise positional score
		lda	sscf_2,x	get castle flag from last move
		sta	sscf,x		save it for this move
		lda	#$ff		enable capture moves
		sta	scap,x		set capture flag
		lda	#63		set FROM position to start address
		sta	sfrm,x
		clr	sdir,x		clear direction
		lda	Ckp		save kings position
		sta	sskp,x
		clr	mflag		init move flag (=0, more moves)
*
* check if there are any moves to make
*
		jsr	next_C_move	try a move
		bcc	S_move_ok	move is ok, continue search
*
* there were no legal moves at all, so if opponents king
* is attacked then opponent is in check mate, otherwise
* we indicate a draw
*
		lda	#draw		init move flag to draw
		sta	mflag
		ldx	Ckp		see if opponent attacked
		jsr	C_attacked
		beq	S_mate		if not attacked, exit with draw score
		rts
S_mate		lda	#mate		otherwise, check mate!
		sta	mflag
S_return	rts

S_try_next	jsr	res_C_move	restore the move
		jsr	next_C_move	get the next move
		bcs	S_return	if no more moves, return

S_move_ok	lda	bal		get material balance
		sub	sstp0		update it with a possible capture
		sta	bal		save it back

		jsr	O_start		do ab search

		ldx	#lb		point to stack
		lda	bal
		add	sstp,x		restore balance
		sta	bal		save it back

		lda	v		get score
		cmp	m,x		compare with current max
		bmi	S_try_next	if less, try next move
*
* get the post positional score for cases:
* 1. the last position searched returned a material balance
*    equal to the maximum material balance, or
* 2. the last position searched returned a material balance
*    greater than the current material balance
*
* These two cases are listed separately because only when
* the material balance from the last position searched and
* the maximum material balance are equal and the positional
* score is a maximum should the new position be considered.
*
		lda	half_moves	are we considering the opening moves?
		cmp	#OP_MOVES
		bhs	S_get_midend	no, get the middle/end game score
		jsr	opening_score	get the opening score
		bra	S_try_best
S_get_midend	jsr	midend_score	get the middle/end game score
S_try_best      ldx	#lb		get the stack pointer
		lda	v		score equal to current max?
		cmp	m,x
		bne	S_newbest	no, must be new max
		lda	mb		get current maximum position score
		cmp	mp		current positional score no better?
		bpl	S_try_next	yes, try the next move
*
* save the best position found so far
*
S_newbest	lda	half_moves	are we considering the opening moves?
		cmp	#OP_MOVES
		bhs	S_save_mid	no, save middle game move
*
* save the best opening move found so far
*
		lda	sfrm,x		FROM position
		sta	pcf0		save for this iteration
		lda	spto,x		TO position
		sta	pct0
		lda	sscf,x		casle flag
		sta	bcf
		lda	Ckp		kings position
		sta	bkp
		ldx	#63		save the board
S_save_board	lda	b,x
		sta	sb,x
		decx
		bpl	S_save_board
		ldx	#lb		get the stack pointer
		bra	S_max		update maximums
*
* save the board after middle game move
*
S_save_mid	lda	sfrm,x		FROM position
		sta	i_frm		save for this iteration
		lda	spto,x		TO position
		sta	i_pto
		lda	ssmf,x		move flag
		sta	i_mf
		lda	scap,x		capture flag
		sta	i_cap
		lda	sscf,x		castle flag
		sta	i_cf
		lda	Ckp		kings position
		sta	i_kp
*
* update the maximums found so far
*
S_max		lda	mp		best positional score
		sta	mb
		lda	v		current maximum material score
		sta	m,x
		cmp	#maxscore	if v is max score, stop searching
		beq	S_exact		yes, exit now
		jmp	S_try_next	no, get next move
S_exact		lda	#know_move	set mflag to non-zero value
		sta	mflag
		jmp	res_C_move	restore the move, and return

*****************************************************************************
*                                                                           *
* Postpositional scoring for opening and middle/end game.                   *
*                                                                           *
*****************************************************************************
*
* calculate the opening position score which is stored in mp
*
opening_score	ldx	spto0		evaluate centre control for this move
		lda	BV,x		get centre control value for TO pos
		ldx	sfrm0		get the FROM position
		sub	BV,x		subtract centre control FROM from TO
		ldx	spto0		get the piece on the move
		ldx	b,x
		stx	temp		save it for later
		ldx	PVO,x		get its value for centre control
		tsta			is difference negative?
		bpl	op_plus		no, a straight multiply
		nega			ensure A is positive
		mul                     this is an unsigned multiply
		nega			ensure result is negative
		bra	op_cont_1	continue
op_plus		mul			do multiply on +ve parameters
op_cont_1       sta	mp		init mp to center control value
*
* having calculated the center control value, determine values for
* opening piece moves
*
		lda	temp		get the saved piece
		cmp	#C_pawn		is it a pawn move
		bne	op_kn		no, check of knight move
*
* score pawn moves to K4 or Q4, advancing a Rook's pawn
*
		lda	sfrm0		get FROM position
		cmp	#8		FROM=8?
		beq	op_pn1		yes, score -10
		cmp	#15		FROM=15?
		bne	op_pn2		no, test next one
op_pn1		lda	#-10		yes, subtract 10 from score
		bra	op_exit

op_pn2		ldx	spto0		get TO position
		cmp	#12		FROM=12?
		bne	op_pn3		no, test next one
		cpx	#28		TO=28?
		bne	op_kside	no, exit
		lda	#30		yes, add 30 to score and return
		bra	op_exit

op_pn3		cmp	#20		FROM=20?
		bne	op_pn4		no, test next one
		cpx	#28		TO=28?
		bne	op_kside	no, exit
		lda	#2		yes, add 2 to score and return
		bra	op_exit

op_pn4		cmp	#11		FROM=11?
		bne	op_pn5		no, test next one
		cpx	#27		TO=27?
		bne	op_kside	no, exit
		lda	#18		yes, add 18 to score and return
		bra	op_exit

op_pn5		cmp	#19		FROM=19?
		bne	op_kside	no, test next one
		cpx	#27		TO=27?
		bne	op_kside	no, exit
		lda	#2		yes, add 2 to score and return
		bra	op_exit
*
* score knight moves to R3
*
op_kn		cmp	#C_knight	is it a knight move
		bne	op_k		no, check if king move
		lda	sfrm0		get FROM position
		ldx	spto0		get TO position
		cmp	#1		FROM=1?
		bne	op_pn6		no, test next one
		cpx	#16		TO=16?
		bne	op_kside	no, exit
		lda	#-15		yes, subtract 15 from score
op_exit		add	mp
		sta	mp
op_rts		rts

op_pn6		cmp	#6		FROM=6?
		bne	op_kside	no, test next one
		cpx	#23		TO=23?
		bne	op_kside	no, exit
		lda	#-15		yes, subtract 15 from score
		bra	op_exit
*
* score castling moves
*
op_k		cmp	#C_king		is it a king move
		bne	op_kside	no, check if king side move
		lda	sfrm0		get FROM position
		cmp	#4		FROM=4 (kings origin)
		bne	op_kside	no, test for king side move
		ldx	spto0		get TO position
		cpx	#6		TO=6? king side castle
		bne	op_k1		no, test queen side
		lda	#30		yes, add 30 to score
		bra	op_exit
op_k1		cpx	#2		queen side castle?
		bne	op_kside	no, test for king side move
		lda	#10		yes, add 10 to score
		bra	op_exit
*
* score king side moves
*
op_kside	lda	sfrm0		get FROM position
		cmp	#5		FROM=5
		beq	op_ks1		yes score +2
		cmp	#6		no, FROM=6?
		bne	op_testblock	no, test block
op_ks1		lda	#2		add 2 to score
		add	mp              and fall through to test for
		sta	mp		pawn blocking
*
* score pawn blocked moves,
* if pawn at 11 or 12 is blocked, score -50
*
op_testblock    lda	b+11		get what's at position 11
		cmp	#C_pawn		a pawn?
		beq	op_bl1		yes, check if blocked
		lda	b+12		get what's at position 12
		cmp	#C_pawn		a pawn?
		bne	op_rts		return with score
		lda     b+20		is there a computer piece here?
op_bl_sc	deca			if blank or opponent result is -ve
		bmi	op_rts
		lda	#-50		pawn is blocked, score -50
		bra     op_exit
op_bl1		lda	b+19		get piece at 19
		bra	op_bl_sc	check it and score it
*
* calculate the middle/end positional score
*
midend_score	clr     mp		init score to 0
		lda	#19		if opponent or computer material
		cmp	Omat		is less than 20 then consider
		bpl	me_end		this the end game and calculate
		cmp	Cmat            mobility only
		bmi	me_middle
me_end		ldx	spto0		get the piece on the move
		lda	b,x
		deca			if pawn, now zero
		bne	me_mobility
		txa			a pawn move, init score
		and	#7		to rank
		sta	mp
		bra	me_mobility
*
* this is the middle game
* score the castling moves
*
me_middle	lda	scap0		if not a capture move, go straight
		bne	me_mobility	to mobility calculation
		lda	ssmf0		test the special move flag
		beq	me_mobility	if not a castle, calc mobility
		bmi	me_qside	-ve indicates queen side castle
		lda	#20		score 30 for king side castle
		bra	me_store
me_qside	lda	#10		score 10 for queen side castle
me_store	sta	mp
*
* calculate mobility, ie number of legal moves available to the
* computer in this position
*
me_mobility	ldx	#lb+2		computer level is even, don't use 0
		stx	l		'cause it's currently in use!
		lda	sfrm0_1		init previous opponent move for ep
		sta	sfrm_1,x
		lda	spto0_1
		sta	spto_1,x
		lda	Ckp		save kings position
		sta	sskp,x
		lda	sscf0_2		init castle flag
		sta	sscf,x
		lda	#$ff		enable capture moves
		sta	scap,x		set capture flag
		lda	#63		init FROM
		sta	sfrm,x
		clr	sdir,x		init DIRection
		bra	me_inloop	count the moves available
me_loop		jsr	res_C_move	restore the board
		inc	mp		count this move
me_inloop	jsr	next_C_move	do a move
		bcc	me_loop		loop if more moves
		lda	#lb		restore l
		sta	l
		rts

*****************************************************************************
*                                                                           *
* MASK OPTION REGISTER                                                      *
*                                                                           *
*****************************************************************************
		ORG     OPTR
		FCB	$80		RAM region 0, IRQ -ve edge only

*****************************************************************************
*                                                                           *
* Set up the interrupt vectors                                              *
*                                                                           *
*****************************************************************************

*****************************************************************************
*                                                                           *
* SPI INTERRUPT                                                             *
*                                                                           *
*****************************************************************************
		ORG     $1FF4
		FDB	reset

*****************************************************************************
*                                                                           *
* SCI INTERRUPT                                                             *
*                                                                           *
*****************************************************************************
		ORG     $1FF6
		FDB	reset
*		FDB	MON_INT

*****************************************************************************
*                                                                           *
* TIMER INTERRUPT                                                           *
*                                                                           *
*****************************************************************************
		ORG     $1FF8
		FDB	timer_int

*****************************************************************************
*                                                                           *
* EXTERNAL INTERRUPT                                                        *
*                                                                           *
*****************************************************************************
		ORG     $1FFA
		FDB	irq_int

*****************************************************************************
*                                                                           *
* SOFTWARE INTERRUPT                                                        *
*                                                                           *
*****************************************************************************
		ORG     $1FFC
		FDB	reset
*		FDB	MON_INT

*****************************************************************************
*                                                                           *
* RESET VECTOR                                                              *
*                                                                           *
*****************************************************************************
		ORG     $1FFE
*		FDB     BEGIN
		FDB	reset

		END
